@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -16,12 +16,8 @@
 #     limitations under the License.
 #
 """ Options module """
-
-version_string = """\
-Nuitka V0.5.21.2
-Copyright (C) 2016 Kay Hayen."""
-
 import logging
+import os
 import sys
 from optparse import SUPPRESS_HELP, OptionGroup, OptionParser
 
@@ -31,16 +27,11 @@ from nuitka.PythonVersions import (
     python_version_str
 )
 from nuitka.utils import Utils
+from nuitka.Version import getNuitkaVersion
 
 # Indicator if we were called as "nuitka-run" in which case we assume some
 # other defaults and work a bit different with parameters.
-is_nuitka_run = Utils.basename(sys.argv[0]).lower().startswith("nuitka-run")
-
-def getVersion():
-    return version_string.split()[1][1:]
-
-def getYear():
-    return int(version_string.split()[4])
+is_nuitka_run = os.path.basename(sys.argv[0]).lower().startswith("nuitka-run")
 
 if not is_nuitka_run:
     usage = "usage: %prog [--module] [--execute] [options] main_module.py"
@@ -49,7 +40,7 @@ else:
 
 parser = OptionParser(
     usage   = usage,
-    version = getVersion()
+    version = getNuitkaVersion()
 )
 
 # This option is obsolete, and module should be used.
@@ -110,6 +101,20 @@ Defaults to what you run Nuitka with (currently %s)""" % (
     )
 )
 
+if os.name == "nt":
+    parser.add_option(
+        "--python-arch",
+        action  = "store",
+        dest    = "python_arch",
+        choices = ("x86", "x86_64"),
+        default = None,
+        help    = """\
+Architecture of Python to use. One of "x86" or "x86_64".
+Defaults to what you run Nuitka with (currently "%s").""" % (
+           Utils.getArchitecture()
+        )
+    )
+
 parser.add_option(
     "--python-debug", "--python-dbg",
     action  = "store_true",
@@ -128,11 +133,21 @@ parser.add_option(
     help    = """\
 Python flags to use. Default uses what you are using to run Nuitka, this
 enforces a specific mode. These are options that also exist to standard
-Python executable. Currently supported: "-S" (alias nosite),
-"static_hashes" (not use Randomization), "no_warnings" (do not give
-Python runtime warnings). Default empty."""
+Python executable. Currently supported: "-S" (alias "nosite"),
+"static_hashes" (not use hash randomization), "no_warnings" (do not give
+Python runtime warnings), "-O" (alias "noasserts"). Default empty."""
 )
 
+parser.add_option(
+    "--python2-for-scons",
+    action  = "store",
+    dest    = "python2_scons",
+    default = None,
+    help    = """\
+If using Python3, provide the path of a Python2 binary to use. For Python2
+defaults to what you run Nuitka with, otherwise searches on Windows in the
+registry or non-Windows in PATH."""
+)
 
 parser.add_option(
     "--warn-implicit-exceptions",
@@ -140,9 +155,17 @@ parser.add_option(
     dest    = "warn_implicit_exceptions",
     default = False,
     help    = """\
-Given warnings for implicit exceptions detected at compile time.""",
+Enable warnings for implicit exceptions detected at compile time.""",
 )
 
+parser.add_option(
+    "--warn-unusual-code",
+    action  = "store_true",
+    dest    = "warn_unusual_code",
+    default = False,
+    help    = """\
+Enable warnings for unusual code detected at compile time.""",
+)
 
 recurse_group = OptionGroup(
     parser,
@@ -300,11 +323,21 @@ codegen_group = OptionGroup(
 codegen_group.add_option(
     "--improved", "--enhanced",
     action  = "store_true",
-    dest    = "improved",
+    dest    = "disabled",
     default = False,
+    help    = SUPPRESS_HELP,
+)
+
+codegen_group.add_option(
+    "--full-compat",
+    action  = "store_false",
+    dest    = "improved",
+    default = True,
     help    = """\
-Allow minor deviations from CPython behavior, e.g. better tracebacks, which
-are not really incompatible, but different.""",
+Enforce absolute compatibility with CPython. Do not even allow minor
+deviations from CPython behavior, e.g. better tracebacks, which are
+not really incompatible, but different. This is intended for tests
+only and should not be necessary for normal use.""",
 )
 
 codegen_group.add_option(
@@ -336,15 +369,6 @@ compatibility reasons, the "__file__" value will always have ".py" suffix
 independent of what it really is."""
 )
 
-codegen_group.add_option(
-    "--no-optimization",
-    action  = "store_true",
-    dest    = "no_optimize",
-    default = False,
-    help    = SUPPRESS_HELP
-# """Disable all unnecessary optimizations on Python level. Defaults to off."""
-)
-
 parser.add_option_group(codegen_group)
 
 outputdir_group = OptionGroup(
@@ -360,7 +384,7 @@ outputdir_group.add_option(
     default = "",
     help    = """\
 Specify where intermediate and final output files should be put. DIRECTORY will
-be populated with C++ files, object files, etc. Defaults to current directory.
+be populated with C files, object files, etc. Defaults to current directory.
 """
 )
 
@@ -437,36 +461,37 @@ Defaults to off."""
 )
 
 debug_group.add_option(
-    "--recompile-c++-only",
+    "--recompile-c-only", "--recompile-c++-only",
     action  = "store_true",
-    dest    = "recompile_cpp_only",
+    dest    = "recompile_c_only",
     default = False,
     help    = """\
-Take existing files and compile them again.Allows compiling edited C++ files
-with the C++ compiler for quick debugging changes to the generated source.
+Take existing files and compile them again. Allows compiling edited C files
+with the C compiler for quick debugging changes to the generated source.
 Defaults to off. Depends on compiling Python source to determine which files it
 should look at."""
 )
 
 debug_group.add_option(
-    "--generate-c++-only",
+    "--generate-c-only",
     action  = "store_true",
-    dest    = "generate_cpp_only",
+    dest    = "generate_c_only",
     default = False,
     help    = """\
-Generate only C++ source code, and do not compile it to binary or module. This
+Generate only C source code, and do not compile it to binary or module. This
 is for debugging and code coverage analysis that doesn't waste CPU. Defaults to
 off."""
 )
 
 debug_group.add_option(
     "--experimental",
-    action  = "store_true",
+    action  = "append",
     dest    = "experimental",
-    default = False,
+    default = [],
     help    = """\
 Use features declared as 'experimental'. May have no effect if no experimental
-features are present in the code. Defaults to off."""
+features are present in the code. Uses secret tags (check source) per
+experimented feature."""
 )
 
 debug_group.add_option(
@@ -491,13 +516,13 @@ parser.add_option(
 
 parser.add_option_group(debug_group)
 
-cpp_compiler_group = OptionGroup(
+c_compiler_group = OptionGroup(
     parser,
-    "Backend C++ compiler choice"
+    "Backend C compiler choice"
 )
 
 
-cpp_compiler_group.add_option(
+c_compiler_group.add_option(
     "--clang",
     action  = "store_true",
     dest    = "clang",
@@ -507,7 +532,7 @@ Enforce the use of clang (needs clang 3.2 or higher).
 Defaults to off."""
 )
 
-cpp_compiler_group.add_option(
+c_compiler_group.add_option(
     "--mingw",
     action  = "store_true",
     dest    = "mingw",
@@ -517,7 +542,7 @@ Enforce the use of MinGW on Windows.
 Defaults to off."""
 )
 
-cpp_compiler_group.add_option(
+c_compiler_group.add_option(
     "--msvc",
     action  = "store",
     dest    = "msvc",
@@ -528,18 +553,18 @@ are e.g. 9.0, 9.0exp, specify an illegal value for a list of installed
 compilers. Defaults to the most recent version."""
 )
 
-cpp_compiler_group.add_option(
+c_compiler_group.add_option(
     "-j", "--jobs",
     action  = "store",
     dest    = "jobs",
     metavar = 'N',
     default = Utils.getCoreCount(),
     help    = """\
-Specify the allowed number of parallel C++ compiler jobs. Defaults to the
+Specify the allowed number of parallel C compiler jobs. Defaults to the
 system CPU count.""",
 )
 
-cpp_compiler_group.add_option(
+c_compiler_group.add_option(
     "--lto",
     action  = "store_true",
     dest    = "lto",
@@ -549,7 +574,7 @@ Use link time optimizations if available and usable (g++ 4.6 and higher).
 Defaults to off."""
 )
 
-parser.add_option_group(cpp_compiler_group)
+parser.add_option_group(c_compiler_group)
 
 tracing_group = OptionGroup(
     parser,
@@ -666,57 +691,108 @@ use. Defaults to off."""
 
 parser.add_option_group(plugin_group)
 
-# First, isolate the first non-option arguments.
-if is_nuitka_run:
-    count = 0
 
-    for count, arg in enumerate(sys.argv):
-        if count == 0:
-            continue
+options = None
+positional_args = None
+extra_args = []
 
-        if arg[0] != '-':
-            break
+def parseArgs():
+    # many cases, pylint: disable=global-statement,too-many-branches
+    global options, positional_args, extra_args
 
-        # Treat "--" as a terminator.
-        if arg == "--":
-            count += 1
-            break
+    # First, isolate the first non-option arguments.
+    if is_nuitka_run:
+        count = 0
 
-    if count > 0:
-        extra_args = sys.argv[count+1:]
-        sys.argv = sys.argv[0:count+1]
-else:
-    extra_args = []
+        for count, arg in enumerate(sys.argv):
+            if count == 0:
+                continue
 
-options, positional_args = parser.parse_args()
+            if arg[0] != '-':
+                break
 
-if not positional_args:
-    parser.print_help()
+            # Treat "--" as a terminator.
+            if arg == "--":
+                count += 1
+                break
 
-    sys.exit("""
-Error, need positional argument with python module or main program.""")
+        if count > 0:
+            extra_args = sys.argv[count+1:]
+            sys.argv = sys.argv[0:count+1]
 
-if not options.immediate_execution and len(positional_args) > 1:
-    parser.print_help()
+    options, positional_args = parser.parse_args()
 
-    sys.exit("""
-Error, need only one positional argument unless "--run" is specified to
-pass them to the compiled program execution.""")
+    if not positional_args:
+        parser.print_help()
 
-if options.verbose:
-    logging.getLogger().setLevel(logging.DEBUG)
-else:
-    logging.getLogger().setLevel(logging.INFO)
+        sys.exit("""
+    Error, need positional argument with python module or main program.""")
 
-# Standalone mode implies an executable, not importing "site" module, which is
-# only for this machine, recursing to all modules, and even including the
-# standard library.
-if options.is_standalone:
-    options.executable = True
-    options.recurse_all = True
+    if not options.immediate_execution and len(positional_args) > 1:
+        parser.print_help()
 
-    if Utils.getOS() == "NetBSD":
-        logging.warning("Standalone mode on NetBSD is not functional, due to $ORIGIN linkage not being supported.")
+        sys.exit("""
+    Error, need only one positional argument unless "--run" is specified to
+    pass them to the compiled program execution.""")
+
+    if options.verbose:
+        logging.getLogger().setLevel(logging.DEBUG)
+    else:
+        logging.getLogger().setLevel(logging.INFO)
+
+    # Standalone mode implies an executable, not importing "site" module, which is
+    # only for this machine, recursing to all modules, and even including the
+    # standard library.
+    if options.is_standalone:
+        if not options.executable:
+            sys.exit("""\
+Error, conflicting options, cannot make standalone module, only executable.""")
+
+        options.recurse_all = True
+
+        if Utils.getOS() == "NetBSD":
+            logging.warning("Standalone mode on NetBSD is not functional, due to $ORIGIN linkage not being supported.")
+
+    for any_case_module in getShallFollowModules():
+        if any_case_module.startswith('.'):
+            bad = True
+        else:
+            for char in "/\\:":
+                if  char in any_case_module:
+                    bad = True
+                    break
+            else:
+                bad = False
+
+        if bad:
+            sys.exit(
+                """\
+Error, '--recurse-to' takes only module names, not directory path '%s'.""" % \
+                any_case_module
+            )
+
+    for no_case_module in getShallFollowInNoCase():
+        if no_case_module.startswith('.'):
+            bad = True
+        else:
+            for char in "/\\:":
+                if  char in no_case_module:
+                    bad = True
+                    break
+            else:
+                bad = False
+
+        if bad:
+            sys.exit(
+                """\
+Error, '--recurse-not-to' takes only module names, not directory path '%s'.""" % \
+                no_case_module
+            )
+
+    scons_python = getPython2PathForScons()
+
+    if scons_python is not None and not os.path.exists(scons_python):
+        sys.exit("Error, no such Python2 binary '%s'." % scons_python)
 
 def shallTraceExecution():
     return options.trace_execution
@@ -733,11 +809,11 @@ def shallDumpBuiltTreeXML():
 def shallDisplayBuiltTree():
     return options.display_tree
 
-def shallOnlyExecCppCall():
-    return options.recompile_cpp_only
+def shallOnlyExecCCompilerCall():
+    return options.recompile_c_only
 
-def shallNotDoExecCppCall():
-    return options.generate_cpp_only
+def shallNotDoExecCCompilerCall():
+    return options.generate_c_only
 
 def shallHaveStatementLines():
     return options.statement_lines
@@ -770,42 +846,9 @@ def getShallFollowModules():
 def isAllowedToReexecute():
     return options.allow_reexecute
 
-for any_case_module in getShallFollowModules():
-    if any_case_module.startswith('.'):
-        bad = True
-    else:
-        for char in "/\\:":
-            if  char in any_case_module:
-                bad = True
-                break
-        else:
-            bad = False
-
-    if bad:
-        sys.exit("""
-Error, '--recurse-to' takes only module names, not directory path '%s'.""" % \
-any_case_module)
-
 def getShallFollowInNoCase():
     return sum([ x.split(',') for x in options.recurse_not_modules ], [])
 
-for no_case_module in getShallFollowInNoCase():
-    if no_case_module.startswith('.'):
-        bad = True
-    else:
-        for char in "/\\:":
-            if  char in no_case_module:
-                bad = True
-                break
-        else:
-            bad = False
-
-    if bad:
-        sys.exit("""
-Error, '--recurse-not-to' takes only module names, not directory path '%s'.""" % \
-no_case_module)
-
-
 def getShallFollowExtra():
     return sum([ x.split(',') for x in options.recurse_extra ], [])
 
@@ -815,15 +858,15 @@ def getShallFollowExtraFilePatterns():
 def shallWarnImplicitRaises():
     return options.warn_implicit_exceptions
 
+def shallWarnUnusualCode():
+    return options.warn_unusual_code
+
 def isDebug():
-    return options.debug or options.debugger
+    return options is not None and (options.debug or options.debugger)
 
 def isPythonDebug():
     return options.python_debug or sys.flags.debug
 
-def isOptimize():
-    return not options.no_optimize
-
 def isUnstripped():
     return options.unstripped or options.profile
 
@@ -835,7 +878,7 @@ def shouldCreateGraph():
 
 def getOutputPath(path):
     if options.output_dir:
-        return Utils.normpath(Utils.joinpath(options.output_dir, path))
+        return os.path.normpath(os.path.join(options.output_dir, path))
     else:
         return path
 
@@ -882,58 +925,97 @@ def isShowProgress():
     return options.show_progress
 
 def isShowMemory():
-    return options.show_memory
+    return options is not None and options.show_memory
+
 
 def isShowInclusion():
     return options.show_inclusion
 
+
 def isRemoveBuildDir():
-    return options.remove_build and not options.generate_cpp_only
+    return options.remove_build and not options.generate_c_only
+
 
 def getIntendedPythonVersion():
     return options.python_version
 
-def isExperimental():
-    return hasattr(options, "experimental") and options.experimental
+
+def getIntendedPythonArch():
+    return options.python_arch
+
+
+def isExperimental(indication):
+    """ Are experimental features to be enabled."""
+
+    return hasattr(options, "experimental") and indication in options.experimental
+
+
+def getExperimentalIndications():
+    if hasattr(options, "experimental"):
+        return options.experimental
+    else:
+        return ()
+
 
 def shallExplainImports():
-    return options.explain_imports
+    return options is not None and options.explain_imports
+
 
 def isStandaloneMode():
     return options.is_standalone
 
+
 def getIconPath():
     return options.icon_path
 
-def getPythonFlags():
-    result = []
 
-    for part in options.python_flags:
-        if part in ("-S", "nosite", "no_site"):
-            result.append("no_site")
-        elif part in ("static_hashes", "norandomization", "no_randomization"):
-            result.append("no_randomization")
-        elif part in ("-v", "trace_imports", "trace_import"):
-            result.append("trace_imports")
-        elif part in ("no_warnings", "nowarnings"):
-            result.append("no_warnings")
-        else:
-            logging.warning("Unsupported flag '%s'.", part)
+def getPythonFlags():
+    result = set()
+
+    for parts in options.python_flags:
+        for part in parts.split(','):
+            if part in ("-S", "nosite", "no_site"):
+                result.add("no_site")
+            elif part in ("static_hashes", "norandomization", "no_randomization"):
+                result.add("no_randomization")
+            elif part in ("-v", "trace_imports", "trace_import"):
+                result.add("trace_imports")
+            elif part in ("no_warnings", "nowarnings"):
+                result.add("no_warnings")
+            elif part in ("-O", "no_asserts", "noasserts"):
+                result.add("no_asserts")
+            else:
+                logging.warning("Unsupported flag '%s'.", part)
 
     return result
 
+
 def shallFreezeAllStdlib():
     return options.freeze_stdlib
 
+
 def getPluginsEnabled():
+    if not options:
+        return ()
+
     return options.plugins_enabled
 
+
 def getPluginsDisabled():
+    if not options:
+        return ()
+
     return options.plugins_disabled
 
+
 def shallDetectMissingPlugins():
-    return options.detect_missing_plugins
+    return options is not None and options.detect_missing_plugins
+
 
 def getPluginOptions(plugin_name):
-    # TODO: This should come from command line, pylint: disable=W0613
+    # TODO: This should come from command line, pylint: disable=unused-argument
     return {}
+
+
+def getPython2PathForScons():
+    return options.python2_scons
