@@ -41,7 +41,7 @@ from .templates.CodeTemplatesModules import (
 
 def _generateCallCodePosOnly(to_name, expression, called_name, called_attribute_name,
                              emit, context):
-    # We have many variants for this to deal with, pylint: disable=R0912
+    # We have many variants for this to deal with, pylint: disable=too-many-branches
 
     assert called_name is not None
     # TODO: Not yet specialized for method calls.
@@ -223,7 +223,7 @@ def _generateCallCodeKwOnly(to_name, expression, call_kw, called_name,
         expression.getCompatibleSourceReference()
     )
 
-    getCallCodeKeywordArgs(
+    _getCallCodeKeywordArgs(
         to_name      = to_name,
         called_name  = called_name,
         call_kw_name = call_kw_name,
@@ -383,7 +383,7 @@ def getInstanceCallCodeNoArgs(to_name, called_name, called_attribute_name,
 
 
 # Outside helper code relies on some quick call to be present.
-quick_calls_used = set([1, 2, 3])
+quick_calls_used = set([1, 2, 3, 4, 5])
 quick_instance_calls_used = set()
 
 
@@ -602,7 +602,7 @@ def getCallCodePosArgs(to_name, called_name, args_name, needs_check, emit,
     context.addCleanupTempName(to_name)
 
 
-def getCallCodeKeywordArgs(to_name, called_name, call_kw_name, emit, context):
+def _getCallCodeKeywordArgs(to_name, called_name, call_kw_name, emit, context):
     emitLineNumberUpdateCode(emit, context)
 
     emit(
