@@ -27,9 +27,13 @@ language syntax.
 """
 
 from nuitka.__past__ import iterItems
-from nuitka.PythonVersions import python_version
 
 from . import Contexts, Emission
+from .AsyncgenCodes import (
+    generateMakeAsyncgenObjectCode,
+    getAsyncgenObjectCode,
+    getAsyncgenObjectDeclCode
+)
 from .AttributeCodes import (
     generateAssignmentAttributeCode,
     generateAttributeLookupCode,
@@ -44,7 +48,9 @@ from .BuiltinCodes import (
     generateBuiltinAnonymousRefCode,
     generateBuiltinBinCode,
     generateBuiltinBoolCode,
-    generateBuiltinBytearrayCode,
+    generateBuiltinBytearray1Code,
+    generateBuiltinBytearray3Code,
+    generateBuiltinClassmethodCode,
     generateBuiltinComplexCode,
     generateBuiltinFloatCode,
     generateBuiltinHexCode,
@@ -54,6 +60,7 @@ from .BuiltinCodes import (
     generateBuiltinRange2Code,
     generateBuiltinRange3Code,
     generateBuiltinRefCode,
+    generateBuiltinStaticmethodCode,
     generateBuiltinSum1Code,
     generateBuiltinSum2Code,
     generateBuiltinType1Code,
@@ -85,7 +92,8 @@ from .CoroutineCodes import (
     generateAsyncNextCode,
     generateAsyncWaitCode,
     generateMakeCoroutineObjectCode,
-    getCoroutineObjectCode
+    getCoroutineObjectCode,
+    getCoroutineObjectDeclCode
 )
 from .DictCodes import (
     generateBuiltinDictCode,
@@ -122,7 +130,6 @@ from .FrameCodes import (
 from .FunctionCodes import (
     generateFunctionCallCode,
     generateFunctionCreationCode,
-    generateFunctionDeclCode,
     generateFunctionOutlineCode,
     getExportScopeCode,
     getFunctionCode,
@@ -130,7 +137,8 @@ from .FunctionCodes import (
 )
 from .GeneratorCodes import (
     generateMakeGeneratorObjectCode,
-    getGeneratorObjectCode
+    getGeneratorObjectCode,
+    getGeneratorObjectDeclCode
 )
 from .GlobalsLocalsCodes import (
     generateBuiltinDir1Code,
@@ -147,7 +155,6 @@ from .Helpers import (
 from .IdCodes import generateBuiltinHashCode, generateBuiltinIdCode
 from .ImportCodes import (
     generateBuiltinImportCode,
-    generateImportModuleCode,
     generateImportModuleHardCode,
     generateImportNameCode,
     generateImportStarCode
@@ -185,13 +192,16 @@ from .OperationCodes import (
     generateOperationUnaryCode
 )
 from .PrintCodes import generatePrintNewlineCode, generatePrintValueCode
-from .RaisingCodes import generateRaiseCode
+from .RaisingCodes import generateRaiseCode, generateRaiseExpressionCode
 from .ReturnCodes import (
-    generateGeneratorReturnCode,
+    generateGeneratorReturnNoneCode,
+    generateGeneratorReturnValueCode,
     generateReturnCode,
+    generateReturnConstantCode,
     generateReturnedValueRefCode
 )
 from .SetCodes import (
+    generateBuiltinFrozensetCode,
     generateBuiltinSetCode,
     generateSetCreationCode,
     generateSetLiteralCreationCode,
@@ -206,6 +216,7 @@ from .SliceCodes import (
 )
 from .StringCodes import (
     generateBuiltinAsciiCode,
+    generateBuiltinBytesCode,
     generateBuiltinChrCode,
     generateBuiltinFormatCode,
     generateBuiltinOrdCode,
@@ -223,6 +234,7 @@ from .TupleCodes import generateBuiltinTupleCode, generateTupleCreationCode
 from .VariableCodes import (
     generateAssignmentVariableCode,
     generateDelVariableCode,
+    generateLocalsDictVariableRefCode,
     generateVariableReferenceCode,
     generateVariableReleaseCode
 )
@@ -244,13 +256,18 @@ def generateFunctionBodyCode(function_body, context):
             parent   = context,
             function = function_body
         )
+    elif function_body.isExpressionClassBody():
+        function_context = Contexts.PythonFunctionDirectContext(
+            parent   = context,
+            function = function_body
+        )
     elif function_body.isExpressionCoroutineObjectBody():
         function_context = Contexts.PythonCoroutineObjectContext(
             parent   = context,
             function = function_body
         )
-    elif function_body.isExpressionClassBody():
-        function_context = Contexts.PythonFunctionDirectContext(
+    elif function_body.isExpressionAsyncgenObjectBody():
+        function_context = Contexts.PythonAsyncgenObjectContext(
             parent   = context,
             function = function_body
         )
@@ -265,24 +282,15 @@ def generateFunctionBodyCode(function_body, context):
             function = function_body
         )
 
-    function_codes = Emission.SourceCodeCollector()
-
-    generateStatementSequenceCode(
-        statement_sequence = function_body.getBody(),
-        allow_none         = True,
-        emit               = function_codes,
-        context            = function_context
-    )
-
     needs_exception_exit = function_body.mayRaiseException(BaseException)
 
     if function_body.isExpressionGeneratorObjectBody():
         function_code = getGeneratorObjectCode(
             context                = function_context,
             function_identifier    = function_identifier,
+            closure_variables      = function_body.getClosureVariables(),
             user_variables         = function_body.getUserLocalVariables(),
             temp_variables         = function_body.getTempVariables(),
-            function_codes         = function_codes.codes,
             needs_exception_exit   = needs_exception_exit,
             needs_generator_return = function_body.needsGeneratorReturnExit()
         )
@@ -290,9 +298,19 @@ def generateFunctionBodyCode(function_body, context):
         function_code = getCoroutineObjectCode(
             context                = function_context,
             function_identifier    = function_identifier,
+            closure_variables      = function_body.getClosureVariables(),
+            user_variables         = function_body.getUserLocalVariables(),
+            temp_variables         = function_body.getTempVariables(),
+            needs_exception_exit   = needs_exception_exit,
+            needs_generator_return = function_body.needsGeneratorReturnExit()
+        )
+    elif function_body.isExpressionAsyncgenObjectBody():
+        function_code = getAsyncgenObjectCode(
+            context                = function_context,
+            function_identifier    = function_identifier,
+            closure_variables      = function_body.getClosureVariables(),
             user_variables         = function_body.getUserLocalVariables(),
             temp_variables         = function_body.getTempVariables(),
-            function_codes         = function_codes.codes,
             needs_exception_exit   = needs_exception_exit,
             needs_generator_return = function_body.needsGeneratorReturnExit()
         )
@@ -304,7 +322,6 @@ def generateFunctionBodyCode(function_body, context):
             closure_variables    = function_body.getClosureVariables(),
             user_variables       = function_body.getUserLocalVariables(),
             temp_variables       = function_body.getTempVariables(),
-            function_codes       = function_codes.codes,
             function_doc         = function_body.getDoc(),
             needs_exception_exit = needs_exception_exit,
             file_scope           = getExportScopeCode(
@@ -321,7 +338,6 @@ def generateFunctionBodyCode(function_body, context):
             closure_variables    = function_body.getClosureVariables(),
             user_variables       = function_body.getUserLocalVariables(),
             temp_variables       = function_body.getTempVariables(),
-            function_codes       = function_codes.codes,
             function_doc         = function_body.getDoc(),
             needs_exception_exit = needs_exception_exit,
             file_scope           = getExportScopeCode(
@@ -332,6 +348,40 @@ def generateFunctionBodyCode(function_body, context):
     return function_code, function_context
 
 
+def _generateFunctionDeclCode(function_body, context):
+    if function_body.isExpressionGeneratorObjectBody():
+        return getGeneratorObjectDeclCode(
+            function_identifier = function_body.getCodeName(),
+        )
+    elif function_body.isExpressionCoroutineObjectBody():
+        return getCoroutineObjectDeclCode(
+            function_identifier = function_body.getCodeName(),
+        )
+    elif function_body.isExpressionAsyncgenObjectBody():
+        return getAsyncgenObjectDeclCode(
+            function_identifier = function_body.getCodeName(),
+        )
+    elif function_body.isExpressionClassBody():
+        return getFunctionDirectDecl(
+            function_identifier = function_body.getCodeName(),
+            closure_variables   = function_body.getClosureVariables(),
+            file_scope          = getExportScopeCode(
+                cross_module = False
+            ),
+            context             = context
+        )
+    elif function_body.needsDirectCall():
+        return getFunctionDirectDecl(
+            function_identifier = function_body.getCodeName(),
+            closure_variables   = function_body.getClosureVariables(),
+            file_scope          = getExportScopeCode(
+                cross_module = function_body.isCrossModuleUsed()
+            ),
+            context             = context
+        )
+    else:
+        return None
+
 
 def prepareModuleCode(global_context, module, module_name):
     # As this not only creates all modules, but also functions, it deals
@@ -373,7 +423,7 @@ def prepareModuleCode(global_context, module, module_name):
 
         function_body_codes.append(function_code)
 
-        function_decl = generateFunctionDeclCode(
+        function_decl = _generateFunctionDeclCode(
             function_body = function_body,
             context       = function_context
 
@@ -423,9 +473,6 @@ def prepareModuleCode(global_context, module, module_name):
         context             = context
     )
 
-    if python_version >= 330:
-        context.getConstantCode("__loader__")
-
     return template_values, context
 
 
@@ -471,13 +518,15 @@ setExpressionDispatchDict(
         "EXPRESSION_BUILTIN_TYPE3"                  : generateBuiltinType3Code,
         "EXPRESSION_BUILTIN_IMPORT"                 : generateBuiltinImportCode,
         "EXPRESSION_BUILTIN_BOOL"                   : generateBuiltinBoolCode,
-        "EXPRESSION_BUILTIN_BYTEARRAY"              : generateBuiltinBytearrayCode,
+        "EXPRESSION_BUILTIN_BYTEARRAY1"             : generateBuiltinBytearray1Code,
+        "EXPRESSION_BUILTIN_BYTEARRAY3"             : generateBuiltinBytearray3Code,
         "EXPRESSION_BUILTIN_INT"                    : generateBuiltinIntCode,
         "EXPRESSION_BUILTIN_LONG"                   : generateBuiltinLongCode,
         "EXPRESSION_BUILTIN_FLOAT"                  : generateBuiltinFloatCode,
         "EXPRESSION_BUILTIN_COMPLEX"                : generateBuiltinComplexCode,
         "EXPRESSION_BUILTIN_LEN"                    : generateBuiltinLenCode,
         "EXPRESSION_BUILTIN_STR"                    : generateBuiltinStrCode,
+        "EXPRESSION_BUILTIN_BYTES"                  : generateBuiltinBytesCode,
         "EXPRESSION_BUILTIN_UNICODE"                : generateBuiltinUnicodeCode,
         "EXPRESSION_BUILTIN_CHR"                    : generateBuiltinChrCode,
         "EXPRESSION_BUILTIN_ORD"                    : generateBuiltinOrdCode,
@@ -487,8 +536,10 @@ setExpressionDispatchDict(
         "EXPRESSION_BUILTIN_TUPLE"                  : generateBuiltinTupleCode,
         "EXPRESSION_BUILTIN_LIST"                   : generateBuiltinListCode,
         "EXPRESSION_BUILTIN_SET"                    : generateBuiltinSetCode,
+        "EXPRESSION_BUILTIN_FROZENSET"              : generateBuiltinFrozensetCode,
         "EXPRESSION_BUILTIN_DICT"                   : generateBuiltinDictCode,
-        "EXPRESSION_BUILTIN_LOCALS"                 : generateBuiltinLocalsCode,
+        "EXPRESSION_BUILTIN_LOCALS_UPDATED"         : generateBuiltinLocalsCode,
+        "EXPRESSION_BUILTIN_LOCALS_COPY"            : generateBuiltinLocalsCode,
         "EXPRESSION_BUILTIN_GLOBALS"                : generateBuiltinGlobalsCode,
         "EXPRESSION_BUILTIN_SUPER"                  : generateBuiltinSuperCode,
         "EXPRESSION_BUILTIN_ISINSTANCE"             : generateBuiltinIsinstanceCode,
@@ -498,6 +549,8 @@ setExpressionDispatchDict(
         "EXPRESSION_BUILTIN_GETATTR"                : generateBuiltinGetattrCode,
         "EXPRESSION_BUILTIN_SETATTR"                : generateBuiltinSetattrCode,
         "EXPRESSION_BUILTIN_OPEN"                   : generateBuiltinOpenCode,
+        "EXPRESSION_BUILTIN_STATICMETHOD"           : generateBuiltinStaticmethodCode,
+        "EXPRESSION_BUILTIN_CLASSMETHOD"            : generateBuiltinClassmethodCode,
         "EXPRESSION_BUILTIN_RANGE1"                 : generateBuiltinRange1Code,
         "EXPRESSION_BUILTIN_RANGE2"                 : generateBuiltinRange2Code,
         "EXPRESSION_BUILTIN_RANGE3"                 : generateBuiltinRange3Code,
@@ -529,11 +582,17 @@ setExpressionDispatchDict(
         "EXPRESSION_CONSTANT_DICT_REF"              : generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_DICT_EMPTY_REF"        : generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_TUPLE_REF"             : generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_TUPLE_EMPTY_REF"       : generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_LIST_REF"              : generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_LIST_EMPTY_REF"        : generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_SET_REF"               : generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_SET_EMPTY_REF"         : generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_FROZENSET_REF"         : generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_FROZENSET_EMPTY_REF"   : generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_SLICE_REF"             : generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_XRANGE_REF"            : generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_TYPE_REF"              : generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_BYTEARRAY_REF"         : generateConstantReferenceCode,
         "EXPRESSION_CONDITIONAL"                    : generateConditionalCode,
         "EXPRESSION_CONDITIONAL_OR"                 : generateConditionalAndOrCode,
         "EXPRESSION_CONDITIONAL_AND"                : generateConditionalAndOrCode,
@@ -548,7 +607,6 @@ setExpressionDispatchDict(
         "EXPRESSION_DICT_OPERATION_NOT_IN"          : generateDictOperationInCode,
         "EXPRESSION_FUNCTION_CREATION"              : generateFunctionCreationCode,
         "EXPRESSION_FUNCTION_CALL"                  : generateFunctionCallCode,
-        "EXPRESSION_IMPORT_MODULE"                  : generateImportModuleCode,
         "EXPRESSION_IMPORT_MODULE_HARD"             : generateImportModuleHardCode,
         "EXPRESSION_IMPORT_NAME"                    : generateImportNameCode,
         "EXPRESSION_LIST_OPERATION_EXTEND"          : generateListOperationExtendCode,
@@ -556,6 +614,7 @@ setExpressionDispatchDict(
         "EXPRESSION_MODULE_FILE_ATTRIBUTE_REF"      : generateModuleFileAttributeCode,
         "EXPRESSION_MAKE_GENERATOR_OBJECT"          : generateMakeGeneratorObjectCode,
         "EXPRESSION_MAKE_COROUTINE_OBJECT"          : generateMakeCoroutineObjectCode,
+        "EXPRESSION_MAKE_ASYNCGEN_OBJECT"           : generateMakeAsyncgenObjectCode,
         "EXPRESSION_MAKE_SET"                       : generateSetCreationCode,
         "EXPRESSION_MAKE_SET_LITERAL"               : generateSetLiteralCreationCode,
         "EXPRESSION_MAKE_TUPLE"                     : generateTupleCreationCode,
@@ -564,6 +623,7 @@ setExpressionDispatchDict(
         "EXPRESSION_OPERATION_BINARY"               : generateOperationBinaryCode,
         "EXPRESSION_OPERATION_BINARY_ADD"           : generateOperationBinaryCode,
         "EXPRESSION_OPERATION_BINARY_MULT"          : generateOperationBinaryCode,
+        "EXPRESSION_OPERATION_BINARY_DIVMOD"        : generateOperationBinaryCode,
         "EXPRESSION_OPERATION_BINARY_INPLACE"       : generateOperationBinaryCode,
         "EXPRESSION_OPERATION_UNARY"                : generateOperationUnaryCode,
         "EXPRESSION_OPERATION_NOT"                  : generateOperationUnaryCode,
@@ -585,6 +645,9 @@ setExpressionDispatchDict(
         "EXPRESSION_STRING_CONCATENATION"           : generateStringContenationCode,
         "EXPRESSION_BUILTIN_FORMAT"                 : generateBuiltinFormatCode,
         "EXPRESSION_BUILTIN_ASCII"                  : generateBuiltinAsciiCode,
+        "EXPRESSION_LOCALS_VARIABLE_REF"            : generateLocalsDictVariableRefCode,
+        "EXPRESSION_RAISE_EXCEPTION"                : generateRaiseExpressionCode,
+
     }
 )
 
@@ -603,7 +666,12 @@ setStatementDispatchDict(
         "STATEMENT_RELEASE_VARIABLE"         : generateVariableReleaseCode,
         "STATEMENT_EXPRESSION_ONLY"          : generateExpressionOnlyCode,
         "STATEMENT_RETURN"                   : generateReturnCode,
-        "STATEMENT_GENERATOR_RETURN"         : generateGeneratorReturnCode,
+        "STATEMENT_RETURN_TRUE"              : generateReturnConstantCode,
+        "STATEMENT_RETURN_FALSE"             : generateReturnConstantCode,
+        "STATEMENT_RETURN_NONE"              : generateReturnConstantCode,
+        "STATEMENT_RETURN_CONSTANT"          : generateReturnConstantCode,
+        "STATEMENT_GENERATOR_RETURN"         : generateGeneratorReturnValueCode,
+        "STATEMENT_GENERATOR_RETURN_NONE"    : generateGeneratorReturnNoneCode,
         "STATEMENT_CONDITIONAL"              : generateBranchCode,
         "STATEMENT_TRY"                      : generateTryCode,
         "STATEMENT_PRINT_VALUE"              : generatePrintValueCode,
