@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -34,20 +34,10 @@ from .LabelCodes import getBranchingCode, getGotoCode, getLabelCode
 
 
 def generateConditionCode(condition, emit, context):
-    # The complexity is needed to avoid unnecessary complex generated C++
-    # pylint: disable=R0914,R0915
+    # The complexity is needed to avoid unnecessary complex generated C
+    # pylint: disable=too-many-locals,too-many-statements
 
-    if condition.isExpressionConstantRef():
-        # TODO: Must not happen, optimization catches this.
-        assert False
-
-        value = condition.getConstant()
-
-        if value:
-            getGotoCode(context.getTrueBranchTarget(), emit)
-        else:
-            getGotoCode(context.getFalseBranchTarget(), emit)
-    elif condition.isExpressionComparison():
+    if condition.isExpressionComparison():
         left_name = context.allocateTempName("compare_left")
 
         generateExpressionCode(
@@ -196,6 +186,12 @@ def generateConditionCode(condition, emit, context):
         )
 
         context.setCurrentSourceCodeReference(old_source_ref)
+    elif condition.isCompileTimeConstant():
+        getBranchingCode(
+            condition = '1' if condition.getCompileTimeConstant() else '0',
+            emit      = emit,
+            context   = context
+        )
     else:
         condition_name = context.allocateTempName("cond_value")
         truth_name = context.allocateTempName("cond_truth", "int")
@@ -251,7 +247,7 @@ def getConditionCheckTrueCode(to_name, value_name, needs_check, emit, context):
 def generateConditionalAndOrCode(to_name, expression, emit, context):
     # This is a complex beast, handling both "or" and "and" expressions,
     # and it needs to micro manage details.
-    # pylint: disable=R0914
+    # pylint: disable=too-many-locals
     if expression.isExpressionConditionalOR():
         prefix = "or_"
     else:
