@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -20,10 +20,69 @@
 from nuitka import Options
 from nuitka.PythonVersions import python_version
 
-from .ConstantCodes import getConstantCode
-from .ErrorCodes import getErrorExitCode, getReleaseCode, getReleaseCodes
-from .GlobalsLocalsCodes import getStoreLocalsCode
+from .ErrorCodes import (
+    getErrorExitBoolCode,
+    getErrorExitCode,
+    getReleaseCode,
+    getReleaseCodes
+)
 from .Helpers import generateExpressionCode
+from .VariableCodes import getVariableAssignmentCode
+
+
+def getStoreLocalsCode(locals_name, variables, is_foreign, emit, context):
+    for variable, version in variables:
+        if not variable.isModuleVariable():
+            key_name = context.getConstantCode(
+                constant = variable.getName()
+            )
+
+            value_name = context.allocateTempName("locals_value", unique = True)
+
+            if is_foreign:
+                emit(
+                   "%s = PyObject_GetItem( %s, %s );" % (
+                       value_name,
+                       locals_name,
+                       key_name,
+                    )
+                )
+
+                getErrorExitBoolCode(
+                    condition = """\
+    %s == NULL && !EXCEPTION_MATCH_BOOL_SINGLE( GET_ERROR_OCCURRED(), PyExc_KeyError )""" % value_name,
+                    emit      = emit,
+                    context   = context
+                )
+
+                emit("CLEAR_ERROR_OCCURRED();")
+
+                context.addCleanupTempName(value_name)
+            else:
+                emit(
+                    "%s = PyDict_GetItem( %s, %s );" % (
+                       value_name,
+                       locals_name,
+                       key_name,
+                    )
+                )
+
+            emit("if ( %s != NULL )" % value_name)
+            emit('{')
+
+            getVariableAssignmentCode(
+                variable      = variable,
+                version       = version,
+                tmp_name      = value_name,
+                needs_release = None, # TODO: Could be known maybe.
+                in_place      = False,
+                emit          = emit,
+                context       = context
+            )
+
+            emit('}')
+
+
 
 
 def generateBuiltinCompileCode(to_name, expression, emit, context):
@@ -193,9 +252,9 @@ def generateExecCode(statement, emit, context):
     globals_arg = statement.getGlobals()
     locals_arg = statement.getLocals()
 
-    source_name = context.allocateTempName("eval_source")
-    globals_name = context.allocateTempName("eval_globals")
-    locals_name = context.allocateTempName("eval_locals")
+    source_name = context.allocateTempName("exec_source")
+    globals_name = context.allocateTempName("exec_globals")
+    locals_name = context.allocateTempName("exec_locals")
 
     generateExpressionCode(
         to_name    = source_name,
@@ -222,14 +281,12 @@ def generateExecCode(statement, emit, context):
 
     # Filename with origin in improved mode.
     if Options.isFullCompat():
-        filename_name = getConstantCode(
-            constant = "<string>",
-            context  = context
+        filename_name = context.getConstantCode(
+            constant = "<string>"
         )
     else:
-        filename_name = getConstantCode(
-            constant = "<string at %s>" % source_ref.getAsString(),
-            context  = context
+        filename_name = context.getConstantCode(
+            constant = "<string at %s>" % source_ref.getAsString()
         )
 
     old_source_ref = context.setCurrentSourceCodeReference(
@@ -244,9 +301,8 @@ def generateExecCode(statement, emit, context):
         to_name           = compiled_name,
         source_name       = source_name,
         filename_name     = filename_name,
-        mode_name         = getConstantCode(
-            constant = "exec",
-            context  = context
+        mode_name         = context.getConstantCode(
+            constant = "exec"
         ),
         flags_name        = "NULL",
         dont_inherit_name = "NULL",
@@ -327,13 +383,11 @@ def _generateEvalCode(to_name, node, emit, context):
         source_name   = source_name,
         globals_name  = globals_name,
         locals_name   = locals_name,
-        filename_name = getConstantCode(
-            constant = filename,
-            context  = context
+        filename_name = context.getConstantCode(
+            constant = filename
         ),
-        mode_name     = getConstantCode(
-            constant = "eval" if node.isExpressionBuiltinEval() else "exec",
-            context  = context
+        mode_name     = context.getConstantCode(
+            constant = "eval" if node.isExpressionBuiltinEval() else "exec"
         ),
         emit          = emit,
         context       = context
@@ -362,7 +416,7 @@ def generateExecfileCode(to_name, expression, emit, context):
 
 def generateLocalsDictSyncCode(statement, emit, context):
     locals_arg = statement.getLocals()
-    locals_name = context.allocateTempName("eval_locals")
+    locals_name = context.allocateTempName("sync_locals")
 
     generateExpressionCode(
         to_name    = locals_name,
@@ -379,7 +433,8 @@ def generateLocalsDictSyncCode(statement, emit, context):
 
     getStoreLocalsCode(
         locals_name = locals_name,
-        provider    = provider,
+        variables   = statement.previous_traces,
+        is_foreign  = provider.hasForeignLocalsDict(),
         emit        = emit,
         context     = context
     )
