@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -28,6 +28,8 @@ them.
 
 from nuitka.PythonVersions import python_version
 
+from .CodeObjectSpecs import CodeObjectSpec
+from .FutureSpecs import fromFlags
 from .StatementNodes import StatementsSequence
 
 
@@ -49,8 +51,7 @@ def checkFrameStatements(value):
 
 
 
-class StatementsFrame(StatementsSequence):
-    kind = "STATEMENTS_FRAME"
+class StatementsFrameBase(StatementsSequence):
 
     checkers = {
         "statements" : checkFrameStatements
@@ -70,9 +71,11 @@ class StatementsFrame(StatementsSequence):
 
         self.needs_frame_exception_preserve = False
 
+    def isStatementsFrame(self):
+        return True
+
     def getDetails(self):
         result = {
-            "guard_mode"  : self.guard_mode,
             "code_object" : self.code_object
         }
 
@@ -81,33 +84,53 @@ class StatementsFrame(StatementsSequence):
         return result
 
     def getDetailsForDisplay(self):
-        result = self.getDetails()
+        result = StatementsSequence.getDetails(self)
+        result.update()
 
         result.update(self.code_object.getDetails())
 
         return result
 
+    @classmethod
+    def fromXML(cls, provider, source_ref, **args):
+        code_object_args = {}
+        other_args = {}
+
+        for key, value in args.items():
+            if key.startswith("co_"):
+                code_object_args[key] = value
+            elif key == "code_flags":
+                code_object_args["future_spec"] = fromFlags(args["code_flags"])
+            else:
+                other_args[key] = value
+
+        code_object = CodeObjectSpec(**code_object_args)
+
+        return cls(
+            code_object = code_object,
+            source_ref  = source_ref,
+            **other_args
+        )
+
     def getGuardMode(self):
         return self.guard_mode
 
     def needsExceptionFramePreservation(self):
         if python_version < 300:
-            preserving = ("full", "once")
+            return self.guard_mode != "generator"
         else:
-            preserving = ("full", "once", "generator")
-
-        return self.guard_mode in preserving
+            return True
 
     def getVarNames(self):
         return self.code_object.getVarNames()
 
     def updateLocalNames(self):
-        """ For use during variable closure phase.
+        """ For use during variable closure phase. Finalize attributes.
 
         """
         provider = self.getParentVariableProvider()
 
-        if provider.isExpressionFunctionBody():
+        if not provider.isCompiledPythonModule():
             self.code_object.updateLocalNames(
                 [
                     variable.getName() for
@@ -116,6 +139,32 @@ class StatementsFrame(StatementsSequence):
                 ]
             )
 
+        is_optimized = not provider.isCompiledPythonModule() and \
+                       not provider.isExpressionClassBody() and \
+                       not provider.hasLocalsDict()
+
+        self.code_object.setFlagIsOptimizedValue(is_optimized)
+
+        new_locals = not provider.isCompiledPythonModule() and \
+                     (python_version < 340 or (
+                     not provider.isExpressionClassBody() and \
+                     not provider.hasLocalsDict()))
+
+        self.code_object.setFlagNewLocalsValue(new_locals)
+
+        if provider.isExpressionGeneratorObjectBody() or \
+           provider.isExpressionCoroutineObjectBody() or \
+           provider.isExpressionAsyncgenObjectBody():
+            closure_provider = provider.getParentVariableProvider()
+        else:
+            closure_provider = provider
+
+        has_closure  = closure_provider.isExpressionFunctionBody() and \
+                       closure_provider.getClosureVariables() != () and \
+                       not closure_provider.isExpressionClassBody()
+
+        self.code_object.setFlagHasClosureValue(has_closure)
+
     def markAsFrameExceptionPreserving(self):
         self.needs_frame_exception_preserve = True
 
@@ -125,37 +174,9 @@ class StatementsFrame(StatementsSequence):
     def getCodeObject(self):
         return self.code_object
 
-    def getCodeObjectHandle(self, context):
-        provider = self.getParentVariableProvider()
-
-        is_optimized = not provider.isCompiledPythonModule() and \
-                       not provider.isExpressionClassBody() and \
-                       not provider.hasLocalsDict()
-
-        new_locals = not provider.isCompiledPythonModule() and \
-                     (python_version < 340 or (
-                     not provider.isExpressionClassBody() and \
-                     not provider.hasLocalsDict()))
-
-        # TODO: Why do this accessing a node, do this outside.
-        return context.getCodeObjectHandle(
-            code_object  = self.code_object,
-            filename     = self.getParentModule().getRunTimeFilename(),
-            line_number  = 1
-                              if provider.isCompiledPythonModule() else
-                            self.source_ref.getLineNumber(),
-            is_optimized = is_optimized,
-            new_locals   = new_locals,
-            has_closure  = provider.isExpressionFunctionBody() and \
-                            provider.getClosureVariables() != () and \
-                            not provider.isExpressionClassBody(),
-            future_flags = provider.getSourceReference().getFutureSpec().\
-                              asFlags()
-        )
-
-    def computeStatementsSequence(self, constraint_collection):
+    def computeStatementsSequence(self, trace_collection):
         # The extraction of parts of the frame that can be moved before or after
-        # the frame scope, takes it toll to complexity, pylint: disable=R0912
+        # the frame scope, takes it toll to complexity, pylint: disable=too-many-branches
         new_statements = []
 
         statements = self.getStatements()
@@ -164,10 +185,10 @@ class StatementsFrame(StatementsSequence):
             # May be frames embedded.
             if statement.isStatementsFrame():
                 new_statement = statement.computeStatementsSequence(
-                    constraint_collection = constraint_collection
+                    trace_collection = trace_collection
                 )
             else:
-                new_statement = constraint_collection.onStatement(
+                new_statement = trace_collection.onStatement(
                     statement = statement
                 )
 
@@ -180,7 +201,7 @@ class StatementsFrame(StatementsSequence):
 
                 if statement is not statements[-1] and \
                    new_statement.isStatementAborting():
-                    constraint_collection.signalChange(
+                    trace_collection.signalChange(
                         "new_statements",
                         statements[count+1].getSourceReference(),
                         "Removed dead statements."
@@ -189,6 +210,12 @@ class StatementsFrame(StatementsSequence):
                     break
 
         if not new_statements:
+            trace_collection.signalChange(
+                "new_statements",
+                self.getSourceReference(),
+                "Removed empty frame object of '%s'." % self.code_object.getCodeObjectName()
+            )
+
             return None
 
         # If our statements changed just now, they are not immediately usable,
@@ -224,7 +251,7 @@ class StatementsFrame(StatementsSequence):
                     node       = self
                 )
             else:
-                constraint_collection.signalChange(
+                trace_collection.signalChange(
                     "new_statements",
                     self.getSourceReference(),
                     "Removed useless frame object of '%s'." % self.code_object.getCodeObjectName()
@@ -239,3 +266,89 @@ class StatementsFrame(StatementsSequence):
                 self.setStatements(new_statements)
 
             return self
+
+
+
+class StatementsFrameModule(StatementsFrameBase):
+    kind = "STATEMENTS_FRAME_MODULE"
+
+    def __init__(self, statements, code_object, source_ref):
+        StatementsFrameBase.__init__(
+            self,
+            statements  = statements,
+            code_object = code_object,
+            guard_mode  = "once",
+            source_ref  = source_ref
+        )
+
+    @staticmethod
+    def hasStructureMember():
+        return False
+
+
+class StatementsFrameFunction(StatementsFrameBase):
+    kind = "STATEMENTS_FRAME_FUNCTION"
+
+    def __init__(self, statements, code_object, source_ref):
+        StatementsFrameBase.__init__(
+            self,
+            statements  = statements,
+            code_object = code_object,
+            guard_mode  = "full",
+            source_ref  = source_ref
+        )
+
+    @staticmethod
+    def hasStructureMember():
+        return False
+
+
+class StatementsFrameGenerator(StatementsFrameBase):
+    kind = "STATEMENTS_FRAME_GENERATOR"
+
+    def __init__(self, statements, code_object, source_ref):
+        StatementsFrameBase.__init__(
+            self,
+            statements  = statements,
+            code_object = code_object,
+            guard_mode  = "generator",
+            source_ref  = source_ref
+        )
+
+    @staticmethod
+    def hasStructureMember():
+        return True
+
+
+class StatementsFrameCoroutine(StatementsFrameBase):
+    kind = "STATEMENTS_FRAME_COROUTINE"
+
+    def __init__(self, statements, code_object, source_ref):
+        StatementsFrameBase.__init__(
+            self,
+            statements  = statements,
+            code_object = code_object,
+            guard_mode  = "generator",
+            source_ref  = source_ref
+        )
+
+    @staticmethod
+    def hasStructureMember():
+        return True
+
+
+class StatementsFrameAsyncgen(StatementsFrameBase):
+    kind = "STATEMENTS_FRAME_ASYNCGEN"
+
+    def __init__(self, statements, code_object, source_ref):
+        StatementsFrameBase.__init__(
+            self,
+            statements  = statements,
+            code_object = code_object,
+            guard_mode  = "generator",
+            source_ref  = source_ref
+        )
+
+    @staticmethod
+    def hasStructureMember():
+        return True
