@@ -30,25 +30,26 @@ classes.
 """
 
 from nuitka import Options, Variables
-from nuitka.nodes.CodeObjectSpecs import CodeObjectSpec
-from nuitka.nodes.FutureSpecs import fromFlags
 from nuitka.optimizations.FunctionInlining import convertFunctionCallToOutline
 from nuitka.PythonVersions import python_version
 from nuitka.tree.Extractions import updateVariableUsage
 
 from .Checkers import checkStatementsSequenceOrNone
+from .CodeObjectSpecs import CodeObjectSpec
+from .ExpressionBases import (
+    CompileTimeConstantExpressionBase,
+    ExpressionBase,
+    ExpressionChildrenHavingBase
+)
+from .FutureSpecs import fromFlags
 from .IndicatorMixins import (
-    MarkLocalsDictIndicator,
-    MarkUnoptimizedFunctionIndicator
+    MarkLocalsDictIndicatorMixin,
+    MarkUnoptimizedFunctionIndicatorMixin
 )
 from .NodeBases import (
     ChildrenHavingMixin,
-    ClosureGiverNodeBase,
+    ClosureGiverNodeMixin,
     ClosureTakerMixin,
-    CompileTimeConstantExpressionMixin,
-    ExpressionChildrenHavingBase,
-    ExpressionMixin,
-    NodeBase,
     SideEffectsFromChildrenMixin
 )
 from .NodeMakingHelpers import (
@@ -59,34 +60,36 @@ from .NodeMakingHelpers import (
 from .ParameterSpecs import ParameterSpec, TooManyArguments, matchCall
 
 
-class ExpressionFunctionBodyBase(ClosureTakerMixin, ChildrenHavingMixin,
-                                 ClosureGiverNodeBase, ExpressionMixin):
+class MaybeLocalVariableUsage(Exception):
+    pass
 
-    def __init__(self, provider, name, code_prefix, is_class, flags, source_ref,
+class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
+                                 ExpressionChildrenHavingBase):
+
+    def __init__(self, provider, name, code_prefix, flags, source_ref,
                  body = None):
+        ExpressionChildrenHavingBase.__init__(
+            self,
+            values     = {
+                "body" : body # Might be None initially in some cases.
+            },
+            source_ref = source_ref
+        )
+
         ClosureTakerMixin.__init__(
             self,
-            provider      = provider,
-            early_closure = is_class
+            provider = provider
         )
 
-        ClosureGiverNodeBase.__init__(
+        ClosureGiverNodeMixin.__init__(
             self,
             name        = name,
-            code_prefix = code_prefix,
-            source_ref  = source_ref
-        )
-
-        ChildrenHavingMixin.__init__(
-            self,
-            values = {
-                "body" : body # Might be None initially in some cases.
-            }
+            code_prefix = code_prefix
         )
 
         # Special things, "has_super" indicates presence of "super" in variable
         # usage, which modifies some behaviors.
-        self.flags = flags
+        self.flags = flags or None
 
         # Hack: This allows some APIs to work although this is not yet
         # officially a child yet. Important during building.
@@ -121,15 +124,29 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ChildrenHavingMixin,
         return None
 
     def hasFlag(self, flag):
-        return flag in self.flags
+        return self.flags is not None and flag in self.flags
 
-    def getLocalsMode(self):
-        if python_version >= 300:
-            return "updated"
-        elif self.isEarlyClosure() or self.isUnoptimized():
-            return "updated"
-        else:
-            return "copy"
+    def discardFlag(self, flag):
+        if self.flags is not None:
+            self.flags.discard(flag)
+
+    @staticmethod
+    def isEarlyClosure():
+        """ Early closure taking means immediate binding of references.
+
+        Normally it's good to lookup name references immediately, but not for
+        functions. In case of a function body it is not allowed to do that,
+        because a later assignment needs to be queried first. Nodes need to
+        indicate via this if they would like to resolve references at the same
+        time as assignments.
+        """
+
+        return False
+
+    def isLocalsUpdatedMode(self):
+        return python_version >= 300 or \
+               self.isEarlyClosure() or \
+               self.isUnoptimized()
 
     def hasVariableName(self, variable_name):
         return variable_name in self.providing or variable_name in self.temp_variables
@@ -139,16 +156,24 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ChildrenHavingMixin,
 
     def getLocalVariables(self):
         return [
-            variable for
-            variable in
+            variable
+            for variable in
+            self.providing.values()
+            if variable.isLocalVariable()
+        ]
+
+    def getLocalVariableNames(self):
+        return [
+            variable.getName()
+            for variable in
             self.providing.values()
             if variable.isLocalVariable()
         ]
 
     def getUserLocalVariables(self):
         return tuple(
-            variable for
-            variable in
+            variable
+            for variable in
             self.providing.values()
             if variable.isLocalVariable() and not variable.isParameterVariable()
             if variable.getOwner() is self
@@ -207,7 +232,7 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ChildrenHavingMixin,
         return result
 
     def getVariableForReference(self, variable_name):
-        # print( "REF func", self, variable_name )
+        # print( "REF func", self.getCodeName(), variable_name )
 
         if self.hasProvidedVariable(variable_name):
             result = self.getProvidedVariable(variable_name)
@@ -221,26 +246,28 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ChildrenHavingMixin,
             if not result.isModuleVariable():
                 self.registerProvidedVariable(result)
 
-            # For "exec" containing/star import containing, we get a
-            # closure variable already, but if it is a module variable,
-            # only then make it a maybe local variable.
-            if not self.isExpressionClassBody() and self.isUnoptimized() and result.isModuleVariable():
-                result = Variables.MaybeLocalVariable(
-                    owner          = self,
-                    maybe_variable = result
-                )
-
-                self.registerProvidedVariable(result)
+            # For "exec" containing/star import containing, we raise this exception to indicate
+            # that instead of merely a variable, to be assigned, we need to replace with locals
+            # dict access.
+            if python_version < 300 and not self.isExpressionClassBody() and \
+               result.isModuleVariable() and \
+               self.isUnoptimized():
+                raise MaybeLocalVariableUsage
 
         return result
 
     def getVariableForClosure(self, variable_name):
-        # print( "getVariableForClosure", self, variable_name )
+        # print( "getVariableForClosure", self.getCodeName(), variable_name, self.isUnoptimized() )
 
         if self.hasProvidedVariable(variable_name):
             return self.getProvidedVariable(variable_name)
-        else:
-            return self.provider.getVariableForClosure(variable_name)
+
+        return self.takeVariableForClosure(variable_name)
+
+    def takeVariableForClosure(self, variable_name):
+        result = self.provider.getVariableForClosure(variable_name)
+        self.taken.add(result)
+        return result
 
     def createProvidedVariable(self, variable_name):
         # print("createProvidedVariable", self, variable_name)
@@ -255,8 +282,10 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ChildrenHavingMixin,
             (names, source_ref)
         )
 
-    def getNonlocalDeclarations(self):
-        return self.non_local_declarations
+    def consumeNonlocalDeclarations(self):
+        result = self.non_local_declarations
+        self.non_local_declarations = ()
+        return result
 
     def getFunctionName(self):
         return self.name
@@ -297,9 +326,9 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ChildrenHavingMixin,
             return self.getBody().mayRaiseException(exception_type)
 
 
-class ExpressionFunctionBody(ExpressionFunctionBodyBase,
-                             MarkLocalsDictIndicator,
-                             MarkUnoptimizedFunctionIndicator):
+class ExpressionFunctionBody(MarkLocalsDictIndicatorMixin,
+                             MarkUnoptimizedFunctionIndicatorMixin,
+                             ExpressionFunctionBodyBase):
     # We really want these many ancestors, as per design, we add properties via
     # base class mix-ins a lot, leading to many methods, pylint: disable=R0901
 
@@ -330,15 +359,14 @@ class ExpressionFunctionBody(ExpressionFunctionBodyBase,
             provider    = provider,
             name        = name,
             code_prefix = "function",
-            is_class    = False,
             flags       = flags,
             body        = body,
             source_ref  = source_ref
         )
 
-        MarkLocalsDictIndicator.__init__(self)
+        MarkLocalsDictIndicatorMixin.__init__(self)
 
-        MarkUnoptimizedFunctionIndicator.__init__(self)
+        MarkUnoptimizedFunctionIndicatorMixin.__init__(self, flags)
 
         self.doc = doc
 
@@ -378,8 +406,6 @@ class ExpressionFunctionBody(ExpressionFunctionBodyBase,
 
         result.update(self.parameters.getDetails())
 
-        result["code_flags"] = ','.join(self.getSourceReference().getFutureSpec().asFlags())
-
         if self.doc is not None:
             result["doc"] = self.doc
 
@@ -529,6 +555,8 @@ class ExpressionFunctionCreation(SideEffectsFromChildrenMixin,
 
         self.code_object = code_object
 
+        self.variable_closure_traces = None
+
     def getName(self):
         return self.getFunctionRef().getName()
 
@@ -548,17 +576,19 @@ class ExpressionFunctionCreation(SideEffectsFromChildrenMixin,
 
     @classmethod
     def fromXML(cls, provider, source_ref, **args):
-        code_object_specs = {}
+        code_object_args = {}
         other_args = {}
 
         for key, value in args.items():
             if key.startswith("co_"):
-                code_object_specs[key] = value
+                code_object_args[key] = value
+            elif key == "code_flags":
+                code_object_args["future_spec"] = fromFlags(args["code_flags"])
             else:
                 other_args[key] = value
 
-        if code_object_specs:
-            code_object = CodeObjectSpec(**code_object_specs)
+        if code_object_args:
+            code_object = CodeObjectSpec(**code_object_args)
         else:
             code_object = None
 
@@ -608,6 +638,14 @@ class ExpressionFunctionCreation(SideEffectsFromChildrenMixin,
 
             return result, "new_raise", "Annotation values contain raise."
 
+        self.variable_closure_traces = []
+
+        for closure_variable in self.getFunctionRef().getFunctionBody().getClosureVariables():
+            trace = trace_collection.getVariableCurrentTrace(closure_variable)
+            trace.addClosureUsage()
+
+            self.variable_closure_traces.append(trace)
+
         # TODO: Function body may know something too.
         return self, None, None
 
@@ -650,15 +688,11 @@ class ExpressionFunctionCreation(SideEffectsFromChildrenMixin,
 
         if call_args is None:
             args_tuple = ()
-        elif call_args.isExpressionConstantRef() or \
-             call_args.isExpressionMakeTuple():
-            args_tuple = call_args.getIterationValues()
         else:
-            # TODO: Can this even happen, i.e. does the above check make
-            # sense.
-            assert False, call_args
+            assert call_args.isExpressionConstantRef() or \
+                   call_args.isExpressionMakeTuple()
 
-            return call_node, None, None
+            args_tuple = call_args.getIterationValues()
 
         function_body = self.getFunctionRef().getFunctionBody()
 
@@ -718,22 +752,25 @@ error""" % self.getName()
 
     def getCallCost(self, values):
         # TODO: Ought to use values. If they are all constant, how about we
-        # assume no cost, pylint: disable=W0613
-
-        if not Options.isExperimental():
-            return None
+        # assume no cost, pylint: disable=unused-argument
 
         function_body = self.getFunctionRef().getFunctionBody()
 
+        if function_body.isExpressionClassBody():
+            if function_body.getBody().getStatements()[0].isStatementReturn():
+                return 0
+
+            return None
+
+        if True or not Options.isExperimental("function_inlining"):
+            return None
+
         if function_body.isExpressionGeneratorObjectBody():
             # TODO: That's not even allowed, is it?
             assert False
 
             return None
 
-        if function_body.isExpressionClassBody():
-            return None
-
         # TODO: Lying for the demo, this is too limiting, but needs frames to
         # be allowed twice in a context.
         if function_body.mayRaiseException(BaseException):
@@ -748,16 +785,24 @@ error""" % self.getName()
             values       = values
         )
 
+    def getClosureVariableVersions(self):
+        return [
+            (trace.getVariable(), trace.getVersion())
+            for trace in self.variable_closure_traces
+        ]
 
 
-class ExpressionFunctionRef(NodeBase, ExpressionMixin):
+
+class ExpressionFunctionRef(ExpressionBase):
     kind = "EXPRESSION_FUNCTION_REF"
 
+    __slots__ = "function_body", "code_name"
+
     def __init__(self, source_ref, function_body = None, code_name = None):
         assert function_body is not None or code_name is not None
         assert code_name != "None"
 
-        NodeBase.__init__(
+        ExpressionBase.__init__(
             self,
             source_ref = source_ref
         )
@@ -868,6 +913,8 @@ class ExpressionFunctionCall(ExpressionChildrenHavingBase):
             source_ref = source_ref
         )
 
+        self.variable_closure_traces = None
+
     def computeExpression(self, trace_collection):
         function = self.getFunction()
 
@@ -876,7 +923,9 @@ class ExpressionFunctionCall(ExpressionChildrenHavingBase):
         # TODO: This needs some design.
         cost = function.getCallCost(values)
 
-        if function.getFunctionRef().getFunctionBody().mayRaiseException(BaseException):
+        function_body = function.getFunctionRef().getFunctionBody()
+
+        if function_body.mayRaiseException(BaseException):
             trace_collection.onExceptionRaiseExit(BaseException)
 
         if cost is not None and cost < 50:
@@ -885,7 +934,19 @@ class ExpressionFunctionCall(ExpressionChildrenHavingBase):
                 values   = values
             )
 
-            return result, "new_statements", "Function call in-lined."
+            return (
+                result,
+                "new_statements",
+                "Function call to '%s' in-lined." % function_body.getCodeName()
+            )
+
+        self.variable_closure_traces = []
+
+        for closure_variable in function_body.getClosureVariables():
+            trace = trace_collection.getVariableCurrentTrace(closure_variable)
+            trace.addClosureUsage()
+
+            self.variable_closure_traces.append(trace)
 
         return self, None, None
 
@@ -906,18 +967,27 @@ class ExpressionFunctionCall(ExpressionChildrenHavingBase):
     getFunction = ExpressionChildrenHavingBase.childGetter("function")
     getArgumentValues = ExpressionChildrenHavingBase.childGetter("values")
 
+    def getClosureVariableVersions(self):
+        return [
+            (trace.getVariable(), trace.getVersion())
+            for trace in self.variable_closure_traces
+        ]
 
 # Needed for Python3.3 and higher
-class ExpressionFunctionQualnameRef(CompileTimeConstantExpressionMixin,
-                                    NodeBase):
+class ExpressionFunctionQualnameRef(CompileTimeConstantExpressionBase):
     kind = "EXPRESSION_FUNCTION_QUALNAME_REF"
+
+    __slots__ = "function_body",
+
     def __init__(self, function_body, source_ref):
-        NodeBase.__init__(self, source_ref = source_ref)
-        CompileTimeConstantExpressionMixin.__init__(self)
+        CompileTimeConstantExpressionBase.__init__(
+            self,
+            source_ref = source_ref
+        )
 
         self.function_body = function_body
 
-    def computeExpression(self, trace_collection):
+    def computeExpressionRaw(self, trace_collection):
         result = makeConstantReplacementNode(
             node     = self,
             constant = self.function_body.getFunctionQualname()
