@@ -50,7 +50,7 @@ from .VariableTraces import (
 signalChange = None
 
 
-class CollectionTracingMixin:
+class CollectionTracingMixin(object):
     def __init__(self):
         # For functions, when we are in here, the currently active one,
         self.variable_actives = {}
@@ -128,7 +128,7 @@ class CollectionTracingMixin:
             self.markActiveVariableAsUnknown(variable)
 
 
-class CollectionStartpointMixin:
+class CollectionStartpointMixin(object):
     def __init__(self):
         # Variable assignments performed in here, last issued number, only used
         # to determine the next number that should be used for a new assignment.
@@ -184,7 +184,7 @@ class CollectionStartpointMixin:
             )
 
     def onExceptionRaiseExit(self, raisable_exceptions, collection = None):
-        # TODO: We might want to track per exception, pylint: disable=W0613
+        # TODO: We might want to track per exception, pylint: disable=unused-argument
 
         if collection is None:
             collection = self
@@ -241,6 +241,7 @@ class CollectionStartpointMixin:
         self.addVariableTrace(variable, version, trace_merge)
 
         return version
+#         return version, trace_merge
 
     def dumpTraces(self):
         debug("Constraint collection state: %s", self)
@@ -339,8 +340,6 @@ class CollectionStartpointMixin:
             result = self._initVariableInit(variable)
         elif variable.isLocalVariable():
             result = self._initVariableUninit(variable)
-        elif variable.isMaybeLocalVariable():
-            result = self._initVariableUnknown(variable)
         elif variable.isModuleVariable():
             result = self._initVariableUnknown(variable)
         elif variable.isTempVariable():
@@ -376,11 +375,11 @@ class TraceCollectionBase(CollectionTracingMixin):
 
     @staticmethod
     def signalChange(tags, source_ref, message):
-        # This is monkey patched from another module, pylint: disable=E1102
+        # This is monkey patched from another module.
         signalChange(tags, source_ref, message)
 
-    def onUsedModule(self, module):
-        return self.parent.onUsedModule(module)
+    def onUsedModule(self, module_name):
+        return self.parent.onUsedModule(module_name)
 
     @staticmethod
     def mustAlias(a, b):
@@ -392,8 +391,9 @@ class TraceCollectionBase(CollectionTracingMixin):
     @staticmethod
     def mustNotAlias(a, b):
         # TODO: not yet really implemented
-        if a.isExpressionConstantDictRef() and b.isExpressionConstantRef():
-            return True
+        if a.isExpressionConstantRef() and b.isExpressionConstantRef():
+            if a.isMutable() or b.isMutable():
+                return True
 
         return False
 
@@ -402,7 +402,7 @@ class TraceCollectionBase(CollectionTracingMixin):
 
     def onControlFlowEscape(self, node):
         # TODO: One day, we should trace which nodes exactly cause a variable
-        # to be considered escaped, pylint: disable=W0613
+        # to be considered escaped, pylint: disable=unused-argument
 
         for variable in self.getActiveVariables():
             if variable.isModuleVariable():
@@ -434,11 +434,10 @@ class TraceCollectionBase(CollectionTracingMixin):
         return self.parent.addVariableMergeMultipleTrace(variable, traces)
 
     def onVariableSet(self, assign_node):
-        variable_ref = assign_node.getTargetVariableRef()
-
-        version = variable_ref.getVariableVersion()
-        variable = variable_ref.getVariable()
+        version = assign_node.getVariableVersion()
+        variable = assign_node.getVariable()
 
+        # TODO: The variable, version and assign_node are redundant to pass.
         variable_trace = VariableTraceAssign(
             owner       = self.owner,
             assign_node = assign_node,
@@ -460,12 +459,9 @@ class TraceCollectionBase(CollectionTracingMixin):
 
         return variable_trace
 
-    def onVariableDel(self, variable_ref):
+    def onVariableDel(self, variable, version):
         # Add a new trace, allocating a new version for the variable, and
         # remember the delete of the current
-        variable = variable_ref.getVariable()
-        version = variable_ref.getVariableVersion()
-
         old_trace = self.getVariableCurrentTrace(variable)
 
         variable_trace = VariableTraceUninit(
@@ -487,26 +483,34 @@ class TraceCollectionBase(CollectionTracingMixin):
 
 
     def onLocalsUsage(self):
-        for variable in self.getActiveVariables():
+        result = []
 
+        if self.owner.isExpressionFunctionBody():
+            include_closure = not self.owner.isUnoptimized()
+        else:
+            include_closure = False
+
+        for variable in self.getActiveVariables():
             # TODO: Currently this is a bit difficult to express in a positive
             # way, but we want to have only local variables.
             if not variable.isTempVariable() and \
-               not variable.isModuleVariable():
+               not variable.isModuleVariable() and \
+               (variable.getOwner() is self.owner or include_closure) and \
+               variable.getName() != ".0":
                 variable_trace = self.getVariableCurrentTrace(
                     variable
                 )
 
                 variable_trace.addNameUsage()
 
-    def onVariableRelease(self, variable):
-        current = self.getVariableCurrentTrace(variable)
-
-        # Annotate that releases to the trace, it may be important knowledge.
-        current.addRelease()
-
-        return current
+                result.append(
+                    (
+                        variable,
+                        variable_trace.getVersion()
+                    )
+                )
 
+        return result
 
     def onVariableContentEscapes(self, variable):
         self.getVariableCurrentTrace(variable).onValueEscape()
@@ -544,17 +548,6 @@ class TraceCollectionBase(CollectionTracingMixin):
                 # Remember the reference for constraint collection.
                 assert new_node.variable_trace.hasDefiniteUsages()
 
-        # We add variable reference nodes late to their traces, only after they
-        # are actually produced, and not resolved to something else, so we do
-        # not have them dangling, and the code complexity inside of their own
-        # "computeExpression" functions.
-        if new_node.isExpressionVariableRef() or \
-           new_node.isExpressionTempVariableRef():
-            if new_node.getVariable().isMaybeLocalVariable():
-                variable_trace = self.getVariableCurrentTrace(
-                    variable = new_node.getVariable().getMaybeVariable()
-                )
-                variable_trace.addUsage()
 
         return new_node
 
@@ -591,26 +584,26 @@ class TraceCollectionBase(CollectionTracingMixin):
 
         # Refuse to do stupid work
         if collection_yes is None and collection_no is None:
-            pass
+            return None
         elif collection_yes is None or collection_no is None:
             # Handle one branch case, we need to merge versions backwards as
             # they may make themselves obsolete.
-            self.mergeMultipleBranches(
+            return self.mergeMultipleBranches(
                 collections = (self, collection_yes or collection_no)
             )
         else:
-            self.mergeMultipleBranches(
-                collections = (collection_yes,collection_no)
+            return self.mergeMultipleBranches(
+                collections = (collection_yes, collection_no)
             )
 
     def mergeMultipleBranches(self, collections):
-        assert len(collections) > 0
+        assert collections
 
         # Optimize for length 1, which is trivial merge and needs not a
         # lot of work.
         if len(collections) == 1:
             self.replaceBranch(collections[0])
-            return
+            return None
 
         variable_versions = {}
 
@@ -628,6 +621,8 @@ class TraceCollectionBase(CollectionTracingMixin):
 
         self.variable_actives = {}
 
+#         merge_traces = None
+
         for variable, versions in iterItems(variable_versions):
             if len(versions) == 1:
                 version, = versions
@@ -641,8 +636,16 @@ class TraceCollectionBase(CollectionTracingMixin):
                     ]
                 )
 
+#                 if merge_traces is None:
+#                     merge_traces = [trace_merge]
+#                 else:
+#                     merge_traces.append(trace_merge)
+
             self.markCurrentVariableTrace(variable, version)
 
+        # Return "None", or turn the list into a tuple for memory savings.
+#         return merge_traces and tuple(merge_traces)
+
     def replaceBranch(self, collection_replace):
         self.variable_actives.update(collection_replace.variable_actives)
         collection_replace.variable_actives = None
@@ -772,17 +775,27 @@ class TraceCollectionFunction(CollectionStartpointMixin,
         assert function_body.isExpressionFunctionBody() or \
                function_body.isExpressionClassBody() or \
                function_body.isExpressionGeneratorObjectBody() or \
-               function_body.isExpressionCoroutineObjectBody(), function_body
+               function_body.isExpressionCoroutineObjectBody() or \
+               function_body.isExpressionAsyncgenObjectBody(), function_body
 
         CollectionStartpointMixin.__init__(self)
 
         TraceCollectionBase.__init__(
             self,
             owner  = function_body,
-            name   = "function_" + str(function_body),
+            name   = "collection_" + function_body.getCodeName(),
             parent = parent
         )
 
+        if function_body.isExpressionFunctionBody():
+            for parameter_variable in function_body.getParameters().getAllVariables():
+                self._initVariableInit(parameter_variable)
+                self.variable_actives[parameter_variable] = 0
+
+        for closure_variable in function_body.getClosureVariables():
+            self._initVariableUnknown(closure_variable)
+            self.variable_actives[closure_variable] = 0
+
 
 class TraceCollectionModule(CollectionStartpointMixin,
                             TraceCollectionBase):
