@@ -49,18 +49,19 @@ catching and passing in exceptions raised.
 
 """
 
+import os
 import sys
-from logging import warning
+from logging import info, warning
 
-from nuitka import Options, SourceCodeReferences, Tracing
-from nuitka.__past__ import long, unicode  # pylint: disable=W0622
+from nuitka import Options, SourceCodeReferences
+from nuitka.__past__ import (  # pylint: disable=I0021,redefined-builtin
+    long,
+    unicode
+)
 from nuitka.importing import Importing
 from nuitka.importing.ImportCache import addImportedModule
 from nuitka.importing.PreloadedPackages import getPthImportedPackages
-from nuitka.nodes.AssignNodes import (
-    ExpressionTargetVariableRef,
-    StatementAssignmentVariable
-)
+from nuitka.nodes.AssignNodes import StatementAssignmentVariableName
 from nuitka.nodes.AttributeNodes import ExpressionAttributeLookup
 from nuitka.nodes.BuiltinFormatNodes import (
     ExpressionBuiltinAscii,
@@ -79,7 +80,6 @@ from nuitka.nodes.ConstantRefNodes import (
 from nuitka.nodes.CoroutineNodes import ExpressionAsyncWait
 from nuitka.nodes.ExceptionNodes import StatementRaiseException
 from nuitka.nodes.GeneratorNodes import StatementGeneratorReturn
-from nuitka.nodes.ImportNodes import ExpressionImportModule
 from nuitka.nodes.LoopNodes import StatementLoopBreak, StatementLoopContinue
 from nuitka.nodes.ModuleNodes import (
     CompiledPythonModule,
@@ -92,19 +92,19 @@ from nuitka.nodes.OperatorNodes import (
     ExpressionOperationUnary,
     makeBinaryOperationNode
 )
-from nuitka.nodes.ReturnNodes import StatementReturn
+from nuitka.nodes.ReturnNodes import (
+    StatementReturn,
+    StatementReturnNone,
+    makeStatementReturnConstant
+)
 from nuitka.nodes.StatementNodes import StatementExpressionOnly
 from nuitka.nodes.StringConcatenationNodes import ExpressionStringConcatenation
-from nuitka.nodes.VariableRefNodes import ExpressionVariableRef
+from nuitka.nodes.VariableRefNodes import ExpressionVariableNameRef
 from nuitka.Options import shallWarnUnusualCode
 from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
-from nuitka.tree.ReformulationForLoopStatements import (
-    buildAsyncForLoopNode,
-    buildForLoopNode
-)
-from nuitka.tree.ReformulationWhileLoopStatements import buildWhileLoopNode
-from nuitka.utils import MemoryUsage, Utils
+from nuitka.utils import MemoryUsage
+from nuitka.utils.FileOperations import splitPath
 
 from . import SyntaxErrors
 from .Helpers import (
@@ -114,6 +114,7 @@ from .Helpers import (
     extractDocFromBody,
     getBuildContext,
     getKind,
+    makeAbsoluteImportNode,
     makeModuleFrame,
     makeStatementsSequence,
     makeStatementsSequenceFromStatement,
@@ -141,6 +142,10 @@ from .ReformulationContractionExpressions import (
 )
 from .ReformulationDictionaryCreation import buildDictionaryNode
 from .ReformulationExecStatements import buildExecNode
+from .ReformulationForLoopStatements import (
+    buildAsyncForLoopNode,
+    buildForLoopNode
+)
 from .ReformulationFunctionStatements import (
     buildAsyncFunctionNode,
     buildFunctionNode
@@ -148,7 +153,10 @@ from .ReformulationFunctionStatements import (
 from .ReformulationImportStatements import (
     buildImportFromNode,
     buildImportModulesNode,
-    checkFutureImportsOnlyAtStart
+    checkFutureImportsOnlyAtStart,
+    getFutureSpec,
+    popFutureSpec,
+    pushFutureSpec
 )
 from .ReformulationLambdaExpressions import buildLambdaNode
 from .ReformulationNamespacePackages import (
@@ -160,6 +168,7 @@ from .ReformulationSequenceCreation import buildSequenceCreationNode
 from .ReformulationSubscriptExpressions import buildSubscriptNode
 from .ReformulationTryExceptStatements import buildTryExceptionNode
 from .ReformulationTryFinallyStatements import buildTryFinallyNode
+from .ReformulationWhileLoopStatements import buildWhileLoopNode
 from .ReformulationWithStatements import buildAsyncWithNode, buildWithNode
 from .ReformulationYieldExpressions import buildYieldFromNode, buildYieldNode
 from .SourceReading import (
@@ -170,7 +179,7 @@ from .VariableClosure import completeVariableClosures
 
 
 def buildVariableReferenceNode(provider, node, source_ref):
-    return ExpressionVariableRef(
+    return ExpressionVariableNameRef(
         variable_name = mangleName(node.id, provider),
         source_ref    = source_ref
     )
@@ -370,6 +379,7 @@ def handleGlobalDeclarationNode(provider, node, source_ref):
             variable = closure_variable
         )
 
+    # Drop this, not really part of our tree.
     return None
 
 
@@ -379,7 +389,8 @@ def handleNonlocalDeclarationNode(provider, node, source_ref):
     parameter_provider = provider
 
     while parameter_provider.isExpressionGeneratorObjectBody() or \
-          parameter_provider.isExpressionCoroutineObjectBody():
+          parameter_provider.isExpressionCoroutineObjectBody() or \
+          parameter_provider.isExpressionAsyncgenObjectBody():
         parameter_provider = parameter_provider.getParentVariableProvider()
 
     if parameter_provider.isExpressionClassBody():
@@ -396,8 +407,12 @@ def handleNonlocalDeclarationNode(provider, node, source_ref):
                 source_ref.atColumnNumber(node.col_offset)
             )
 
-    provider.addNonlocalsDeclaration(node.names, source_ref.atColumnNumber(node.col_offset))
+    provider.addNonlocalsDeclaration(
+        names      = node.names,
+        source_ref = source_ref.atColumnNumber(node.col_offset)
+    )
 
+    # Drop this, not really part of our tree.
     return None
 
 
@@ -437,10 +452,10 @@ def buildEllipsisNode(source_ref):
 
 
 def buildStatementLoopContinue(node, source_ref):
-    # Python forbids this, although technically it's probably not much of
-    # an issue.
     source_ref = source_ref.atColumnNumber(node.col_offset)
 
+    # Python forbids this, although technically it's probably not much of
+    # an issue.
     if getBuildContext() == "finally":
         SyntaxErrors.raiseSyntaxError(
             "'continue' not supported inside 'finally' clause",
@@ -454,7 +469,7 @@ def buildStatementLoopContinue(node, source_ref):
 
 def buildStatementLoopBreak(provider, node, source_ref):
     # A bit unusual, we need the provider, but not the node,
-    # pylint: disable=W0613
+    # pylint: disable=unused-argument
 
     return StatementLoopBreak(
         source_ref = source_ref.atColumnNumber(node.col_offset)
@@ -485,22 +500,41 @@ def buildReturnNode(provider, node, source_ref):
                 source_ref.atColumnNumber(node.col_offset)
             )
 
-    if expression is None:
-        expression = ExpressionConstantNoneRef(
-            source_ref    = source_ref,
-            user_provided = True
-        )
+    if provider.isExpressionAsyncgenObjectBody():
+        if expression is not None:
+            SyntaxErrors.raiseSyntaxError(
+                "'return' with value in async generator",
+                source_ref.atColumnNumber(node.col_offset)
+            )
+
+
+    if provider.isExpressionGeneratorObjectBody() or \
+       provider.isExpressionAsyncgenObjectBody():
+        if expression is None:
+            expression = ExpressionConstantNoneRef(
+                source_ref    = source_ref,
+                user_provided = True
+            )
 
-    if provider.isExpressionGeneratorObjectBody():
         return StatementGeneratorReturn(
             expression = expression,
             source_ref = source_ref
         )
     else:
-        return StatementReturn(
-            expression = expression,
-            source_ref = source_ref
-        )
+        if expression is None:
+            return StatementReturnNone(
+                source_ref = source_ref
+            )
+        elif expression.isExpressionConstantRef():
+            return makeStatementReturnConstant(
+                constant   = expression.getCompileTimeConstant(),
+                source_ref = source_ref
+            )
+        else:
+            return StatementReturn(
+                expression = expression,
+                source_ref = source_ref
+            )
 
 
 def buildExprOnlyNode(provider, node, source_ref):
@@ -534,8 +568,9 @@ def buildUnaryOpNode(provider, node, source_ref):
 def buildBinaryOpNode(provider, node, source_ref):
     operator = getKind(node.op)
 
-    if operator == "Div" and source_ref.getFutureSpec().isFutureDivision():
-        operator = "TrueDiv"
+    if operator == "Div":
+        if getFutureSpec().isFutureDivision():
+            operator = "TrueDiv"
 
     left       = buildNode(provider, node.left, source_ref)
     right      = buildNode(provider, node.right, source_ref)
@@ -690,7 +725,11 @@ setBuildingDispatchers(
 
 def buildParseTree(provider, source_code, source_ref, is_module, is_main):
     # There are a bunch of branches here, mostly to deal with version
-    # differences for module default variables.
+    # differences for module default variables. pylint: disable=too-many-branches
+
+    pushFutureSpec()
+    if is_module:
+        provider.future_spec = getFutureSpec()
 
     body = parseSourceCodeToAst(
         source_code = source_code,
@@ -699,6 +738,9 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
     )
     body, doc = extractDocFromBody(body)
 
+    if is_module and is_main and python_version >= 360:
+        provider.markAsNeedsAnnotationsDictionary()
+
     result = buildStatementsNode(
         provider   = provider,
         nodes      = body,
@@ -718,10 +760,8 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
             for path_imported_name in getPthImportedPackages():
                 statements.append(
                     StatementExpressionOnly(
-                        expression = ExpressionImportModule(
+                        expression = makeAbsoluteImportNode(
                             module_name = path_imported_name,
-                            import_list = (),
-                            level       = 0,
                             source_ref  = source_ref,
                         ),
                         source_ref = source_ref
@@ -730,10 +770,8 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
 
             statements.append(
                 StatementExpressionOnly(
-                    expression = ExpressionImportModule(
+                    expression = makeAbsoluteImportNode(
                         module_name = "site",
-                        import_list = (),
-                        level       = 0,
                         source_ref  = source_ref,
                     ),
                     source_ref = source_ref
@@ -741,30 +779,24 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
             )
 
         statements.append(
-            StatementAssignmentVariable(
-                variable_ref = ExpressionTargetVariableRef(
-                    variable_name = "__doc__",
-                    source_ref    = internal_source_ref
-                ),
-                source       = makeConstantRefNode(
+            StatementAssignmentVariableName(
+                variable_name = "__doc__",
+                source        = makeConstantRefNode(
                     constant      = doc,
                     source_ref    = internal_source_ref,
                     user_provided = True
                 ),
-                source_ref   = internal_source_ref
+                source_ref    = internal_source_ref
             )
         )
 
         statements.append(
-            StatementAssignmentVariable(
-                variable_ref = ExpressionTargetVariableRef(
-                    variable_name = "__file__",
-                    source_ref    = internal_source_ref
-                ),
-                source       = ExpressionModuleFileAttributeRef(
+            StatementAssignmentVariableName(
+                variable_name = "__file__",
+                source        = ExpressionModuleFileAttributeRef(
                     source_ref = internal_source_ref,
                 ),
-                source_ref   = internal_source_ref
+                source_ref    = internal_source_ref
             )
         )
 
@@ -776,16 +808,13 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
 
     if python_version >= 300:
         statements.append(
-            StatementAssignmentVariable(
-                variable_ref = ExpressionTargetVariableRef(
-                    variable_name = "__cached__",
-                    source_ref    = internal_source_ref
-                ),
-                source       = ExpressionConstantNoneRef(
+            StatementAssignmentVariableName(
+                variable_name = "__cached__",
+                source        = ExpressionConstantNoneRef(
                     source_ref    = internal_source_ref,
                     user_provided = True
                 ),
-                source_ref   = internal_source_ref
+                source_ref    = internal_source_ref
             )
         )
 
@@ -793,19 +822,16 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
     if python_version >= 330:
         # For Python3.3, it's set for both packages and non-packages.
         statements.append(
-            StatementAssignmentVariable(
-                variable_ref = ExpressionTargetVariableRef(
-                    variable_name = "__package__",
-                    source_ref    = internal_source_ref
-                ),
-                source       = makeConstantRefNode(
+            StatementAssignmentVariableName(
+                variable_name = "__package__",
+                source        = makeConstantRefNode(
                     constant      = provider.getFullName()
                                       if provider.isCompiledPythonPackage() else
                                     provider.getPackage(),
                     source_ref    = internal_source_ref,
                     user_provided = True
                 ),
-                source_ref   = internal_source_ref
+                source_ref    = internal_source_ref
             )
         )
 
@@ -815,34 +841,28 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
     if needs__initializing__:
         # Set "__initializing__" at the beginning to True
         statements.append(
-            StatementAssignmentVariable(
-                variable_ref = ExpressionTargetVariableRef(
-                    variable_name = "__initializing__",
-                    source_ref    = internal_source_ref
-                ),
-                source       = makeConstantRefNode(
+            StatementAssignmentVariableName(
+                variable_name = "__initializing__",
+                source        = makeConstantRefNode(
                     constant      = True,
                     source_ref    = internal_source_ref,
                     user_provided = True
                 ),
-                source_ref   = internal_source_ref
+                source_ref    = internal_source_ref
             )
         )
 
-    if python_version >= 360:
+    if provider.needsAnnotationsDictionary():
         # Set "__annotations__" on module level to {}
         statements.append(
-            StatementAssignmentVariable(
-                variable_ref = ExpressionTargetVariableRef(
-                    variable_name = "__annotations__",
-                    source_ref    = internal_source_ref
-                ),
-                source       = makeConstantRefNode(
+            StatementAssignmentVariableName(
+                variable_name = "__annotations__",
+                source        = makeConstantRefNode(
                     constant      = {},
                     source_ref    = internal_source_ref,
                     user_provided = True
                 ),
-                source_ref   = internal_source_ref
+                source_ref    = internal_source_ref
             )
         )
 
@@ -856,44 +876,45 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
     if needs__initializing__:
         # Set "__initializing__" at the end to False
         statements.append(
-            StatementAssignmentVariable(
-                variable_ref = ExpressionTargetVariableRef(
-                    variable_name = "__initializing__",
-                    source_ref    = internal_source_ref
-                ),
-                source       = makeConstantRefNode(
+            StatementAssignmentVariableName(
+                variable_name = "__initializing__",
+                source        = makeConstantRefNode(
                     constant      = False,
                     source_ref    = internal_source_ref,
                     user_provided = True
                 ),
-                source_ref   = internal_source_ref
+                source_ref    = internal_source_ref
             )
         )
 
 
     if is_module:
-        return makeModuleFrame(
+        result = makeModuleFrame(
             module     = provider,
             statements = statements,
             source_ref = source_ref
         )
+
+        popFutureSpec()
+
+        return result
     else:
         assert False
 
 
 def decideModuleTree(filename, package, is_shlib, is_top, is_main):
-    # Many variables, branches, due to the many cases, pylint: disable=R0912,R0915
+    # Many variables, branches, due to the many cases, pylint: disable=too-many-branches,too-many-statements
 
     assert package is None or type(package) is str
     assert filename is not None
 
-    if is_main and Utils.isDir(filename):
-        source_filename = Utils.joinpath(filename, "__main__.py")
+    if is_main and os.path.isdir(filename):
+        source_filename = os.path.join(filename, "__main__.py")
 
-        if not Utils.isFile(source_filename):
+        if not os.path.isfile(source_filename):
             sys.stderr.write(
                 "%s: can't find '__main__' module in '%s'\n" % (
-                    Utils.basename(sys.argv[0]),
+                    os.path.basename(sys.argv[0]),
                     filename
                 )
             )
@@ -905,7 +926,7 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
     else:
         main_added = False
 
-    if Utils.isFile(filename):
+    if os.path.isfile(filename):
         source_filename = filename
 
         source_ref = SourceCodeReferences.fromFilename(
@@ -915,7 +936,7 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
         if is_main:
             module_name = "__main__"
         else:
-            module_name = Utils.basename(filename)
+            module_name = os.path.basename(filename)
 
             if module_name.endswith(".py"):
                 module_name = module_name[:-3]
@@ -940,9 +961,10 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
             )
         elif is_main:
             result = PythonMainModule(
-                main_added = main_added,
-                mode       = Plugins.decideCompilation("__main__", source_ref),
-                source_ref = source_ref
+                main_added  = main_added,
+                mode        = Plugins.decideCompilation("__main__", source_ref),
+                future_spec = None,
+                source_ref  = source_ref
             )
         else:
             if package is not None:
@@ -954,18 +976,19 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
                 name         = module_name,
                 package_name = package,
                 mode         = Plugins.decideCompilation(full_name, source_ref),
+                future_spec  = None,
                 source_ref   = source_ref
             )
 
     elif Importing.isPackageDir(filename):
         if is_top:
-            package_name = Utils.splitpath(filename)[-1]
+            package_name = splitPath(filename)[-1]
         else:
-            package_name = Utils.basename(filename)
+            package_name = os.path.basename(filename)
 
-        source_filename = Utils.joinpath(filename, "__init__.py")
+        source_filename = os.path.join(filename, "__init__.py")
 
-        if not Utils.isFile(source_filename):
+        if not os.path.isfile(source_filename):
             source_ref, result = createNamespacePackage(
                 package_name   = package_name,
                 module_relpath = filename
@@ -973,7 +996,7 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
             source_filename = None
         else:
             source_ref = SourceCodeReferences.fromFilename(
-                filename = Utils.abspath(source_filename),
+                filename = os.path.abspath(source_filename),
             )
 
             if package is not None:
@@ -985,12 +1008,15 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
                 name         = package_name,
                 package_name = package,
                 mode         = Plugins.decideCompilation(full_name, source_ref),
+                future_spec  = None,
                 source_ref   = source_ref
             )
+
+            assert result.getFullName() == full_name, result
     else:
         sys.stderr.write(
             "%s: can't open file '%s'.\n" % (
-                Utils.basename(sys.argv[0]),
+                os.path.basename(sys.argv[0]),
                 filename
             )
         )
@@ -1047,7 +1073,7 @@ def createModuleTree(module, source_ref, source_code, is_main):
     if Options.isShowMemory():
         memory_watch.finish()
 
-        Tracing.printLine(
+        info(
             "Memory usage changed loading module '%s': %s" % (
                 module.getFullName(),
                 memory_watch.asStr()
