@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,12 +22,13 @@
 import ast
 from logging import warning
 
-from nuitka import Constants, Tracing
+from nuitka import Constants, Options, Tracing
+from nuitka.nodes.CodeObjectSpecs import CodeObjectSpec
 from nuitka.nodes.ConditionalNodes import StatementConditional
-from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
 from nuitka.nodes.ContainerMakingNodes import (
     ExpressionMakeList,
-    ExpressionMakeSet,
+    ExpressionMakeSetLiteral,
     ExpressionMakeTuple
 )
 from nuitka.nodes.DictionaryNodes import (
@@ -35,13 +36,21 @@ from nuitka.nodes.DictionaryNodes import (
     ExpressionMakeDict
 )
 from nuitka.nodes.ExceptionNodes import StatementRaiseException
-from nuitka.nodes.FrameNodes import StatementsFrame
+from nuitka.nodes.FrameNodes import (
+    StatementsFrameAsyncgen,
+    StatementsFrameCoroutine,
+    StatementsFrameFunction,
+    StatementsFrameGenerator,
+    StatementsFrameModule
+)
+from nuitka.nodes.ImportNodes import ExpressionBuiltinImport
 from nuitka.nodes.NodeBases import NodeBase
 from nuitka.nodes.NodeMakingHelpers import mergeStatements
 from nuitka.nodes.OperatorNodes import ExpressionOperationNOT
-from nuitka.nodes.StatementNodes import (
-    StatementGeneratorEntry,
-    StatementsSequence
+from nuitka.nodes.StatementNodes import StatementsSequence
+from nuitka.PythonVersions import (
+    needsSetLiteralReverseInsertion,
+    python_version
 )
 
 
@@ -54,11 +63,168 @@ def getKind(node):
 
 
 def extractDocFromBody(node):
+    body = node.body
+    doc = None
+
     # Work around ast.get_docstring breakage.
-    if len(node.body) > 0 and getKind(node.body[0]) == "Expr" and getKind(node.body[0].value) == "Str":
-        return node.body[1:], node.body[0].value.s
+    if node.body and \
+       getKind(node.body[0]) == "Expr" and \
+       getKind(node.body[0].value) == "Str":
+
+        if "no_asserts" not in Options.getPythonFlags():
+            doc = body[0].value.s
+
+        body = body[1:]
+
+    return body, doc
+
+
+def parseSourceCodeToAst(source_code, filename, line_offset):
+    # Workaround: ast.parse cannot cope with some situations where a file is not
+    # terminated by a new line.
+    if not source_code.endswith('\n'):
+        source_code = source_code + '\n'
+
+    body = ast.parse(source_code, filename)
+    assert getKind(body) == "Module"
+
+    if line_offset > 0:
+        ast.increment_lineno(body, line_offset)
+
+    return body
+
+
+def detectFunctionBodyKind(nodes, start_value = None):
+    # This is a complex mess, following the scope means a lot of checks need
+    # to be done. pylint: disable=too-many-branches,too-many-statements
+
+    indications = set()
+    if start_value is not None:
+        indications.add(start_value)
+
+    flags = set()
+
+    def _check(node):
+        node_class = node.__class__
+
+        if node_class is ast.Yield:
+            indications.add("Generator")
+        elif python_version >= 330 and node_class is ast.YieldFrom:  # @UndefinedVariable
+            indications.add("Generator")
+        elif python_version >= 350 and node_class in (ast.Await, ast.AsyncWith):  # @UndefinedVariable
+            indications.add("Coroutine")
+
+        # Recurse to children, but do not cross scope boundary doing so.
+        if node_class is ast.ClassDef:
+            for name, field in ast.iter_fields(node):
+                if name in ("name", "body"):
+                    pass
+                elif name in ("bases", "decorator_list", "keywords"):
+                    for child in field:
+                        _check(child)
+                elif name == "starargs":
+                    if field is not None:
+                        _check(field)
+                elif name == "kwargs":
+                    if field is not None:
+                        _check(field)
+                else:
+                    assert False, (name, field, ast.dump(node))
+        elif node_class in (ast.FunctionDef, ast.Lambda) or \
+             (python_version >= 350 and node_class is ast.AsyncFunctionDef):  # @UndefinedVariable
+            for name, field in ast.iter_fields(node):
+                if name in ("name", "body"):
+                    pass
+                elif name in ("bases", "decorator_list"):
+                    for child in field:
+                        _check(child)
+                elif name == "args":
+                    for child in field.defaults:
+                        _check(child)
+
+                    if python_version >= 300:
+                        for child in node.args.kw_defaults:
+                            if child is not None:
+                                _check(child)
+
+                        for child in node.args.args:
+                            if child.annotation is not None:
+                                _check(child.annotation)
+
+                elif name == "returns":
+                    if field is not None:
+                        _check(field)
+                else:
+                    assert False, (name, field, ast.dump(node))
+        elif node_class is ast.GeneratorExp:
+            for name, field in ast.iter_fields(node):
+                if name in ("name", "body", "comparators", "elt"):
+                    pass
+                elif name == "generators":
+                    _check(field[0].iter)
+                else:
+                    assert False, (name, field, ast.dump(node))
+        elif node_class is ast.ListComp and python_version >= 300:
+            for name, field in ast.iter_fields(node):
+                if name in ("name", "body", "comparators"):
+                    pass
+                elif name == "generators":
+                    _check(field[0].iter)
+                elif name in("body", "elt"):
+                    _check(field)
+                else:
+                    assert False, (name, field, ast.dump(node))
+        elif python_version >= 270 and node_class is ast.SetComp:  # @UndefinedVariable
+            for name, field in ast.iter_fields(node):
+                if name in ("name", "body", "comparators", "elt"):
+                    pass
+                elif name == "generators":
+                    _check(field[0].iter)
+                else:
+                    assert False, (name, field, ast.dump(node))
+        elif python_version >= 270 and node_class is ast.DictComp:  # @UndefinedVariable
+            for name, field in ast.iter_fields(node):
+                if name in ("name", "body", "comparators", "key", "value"):
+                    pass
+                elif name == "generators":
+                    _check(field[0].iter)
+                else:
+                    assert False, (name, field, ast.dump(node))
+        elif node_class is ast.Name:
+            if python_version >= 300 and node.id == "super":
+                flags.add("has_super")
+        elif python_version < 300 and node_class is ast.Exec:
+            flags.add("has_exec")
+
+            if node.globals is None:
+                flags.add("has_unqualified_exec")
+
+            for child in ast.iter_child_nodes(node):
+                _check(child)
+        elif python_version < 300 and node_class is ast.ImportFrom:
+            for import_desc in node.names:
+                if import_desc.name[0] == '*':
+                    flags.add("has_exec")
+            for child in ast.iter_child_nodes(node):
+                _check(child)
+        else:
+            for child in ast.iter_child_nodes(node):
+                _check(child)
+
+    for node in nodes:
+        _check(node)
+
+    if indications:
+        if "Coroutine" in indications and "Generator" in indications:
+            function_kind = "Asyncgen"
+        else:
+            # If we found something, make sure we agree on all clues.
+            assert len(indications) == 1, indications
+            function_kind = indications.pop()
     else:
-        return node.body, None
+        function_kind = "Function"
+
+    return function_kind, flags
 
 
 build_nodes_args3 = None
@@ -67,7 +233,7 @@ build_nodes_args1 = None
 
 def setBuildingDispatchers(path_args3, path_args2, path_args1):
     # Using global here, as this is really a singleton, in the form of a module,
-    # and this is to break the cyclic dependency it has, pylint: disable=W0603
+    # and this is to break the cyclic dependency it has, pylint: disable=global-statement
 
     global build_nodes_args3, build_nodes_args2, build_nodes_args1
 
@@ -106,7 +272,7 @@ def buildNode(provider, node, source_ref, allow_none = False):
         elif kind == "Pass":
             result = None
         else:
-            assert False, kind
+            assert False, ast.dump(node)
 
         if result is None and allow_none:
             return None
@@ -116,6 +282,10 @@ def buildNode(provider, node, source_ref, allow_none = False):
         return result
     except SyntaxError:
         raise
+    except RuntimeError:
+        # Very likely the stack overflow, which we will turn into too complex
+        # code exception, don't warn about it with a code dump then.
+        raise
     except:
         warning("Problem at '%s' with %s." % (source_ref, ast.dump(node)))
         raise
@@ -142,27 +312,53 @@ def buildNodeList(provider, nodes, source_ref, allow_none = False):
 
 
 def makeModuleFrame(module, statements, source_ref):
-    assert module.isPythonModule()
+    assert module.isCompiledPythonModule()
 
     if module.isMainModule():
         code_name = "<module>"
     else:
         code_name = module.getName()
 
-    return StatementsFrame(
-        statements    = statements,
-        guard_mode    = "once",
-        var_names     = (),
-        arg_count     = 0,
-        kw_only_count = 0,
-        code_name     = code_name,
-        has_starlist  = False,
-        has_stardict  = False,
-        source_ref    = source_ref
+    return StatementsFrameModule(
+        statements  = statements,
+        code_object = CodeObjectSpec(
+            co_name           = code_name,
+            co_kind           = "Module",
+            co_varnames       = (),
+            co_argcount       = 0,
+            co_kwonlyargcount = 0,
+            co_has_starlist   = False,
+            co_has_stardict   = False,
+            co_filename       = module.getRunTimeFilename(),
+            co_lineno         = source_ref.getLineNumber(),
+            future_spec       = module.getFutureSpec()
+        ),
+        source_ref  = source_ref
     )
 
 
-def buildStatementsNode(provider, nodes, source_ref, frame = False):
+def buildStatementsNode(provider, nodes, source_ref):
+    # We are not creating empty statement sequences.
+    if nodes is None:
+        return None
+
+    # Build as list of statements, throw away empty ones, and remove useless
+    # nesting.
+    statements = buildNodeList(provider, nodes, source_ref, allow_none = True)
+    statements = mergeStatements(statements)
+
+    # We are not creating empty statement sequences. Might be empty, because
+    # e.g. a global node generates not really a statement, or pass statements.
+    if not statements:
+        return None
+    else:
+        return StatementsSequence(
+            statements = statements,
+            source_ref = source_ref
+        )
+
+
+def buildFrameNode(provider, nodes, code_object, source_ref):
     # We are not creating empty statement sequences.
     if nodes is None:
         return None
@@ -177,48 +373,35 @@ def buildStatementsNode(provider, nodes, source_ref, frame = False):
     if not statements:
         return None
 
-    # In case of a frame is desired, build it instead.
-    if frame:
-        if provider.isExpressionFunctionBody():
-            parameters = provider.getParameters()
-
-            arg_names     = parameters.getCoArgNames()
-            kw_only_count = parameters.getKwOnlyParameterCount()
-            code_name     = provider.getFunctionName()
-            guard_mode    = "generator" if provider.isGenerator() else "full"
-            has_starlist  = parameters.getStarListArgumentName() is not None
-            has_stardict  = parameters.getStarDictArgumentName() is not None
-
-            if provider.isGenerator():
-                statements.insert(
-                    0,
-                    StatementGeneratorEntry(
-                        source_ref = source_ref
-                    )
-                )
-
-            return StatementsFrame(
-                statements    = statements,
-                guard_mode    = guard_mode,
-                var_names     = arg_names,
-                arg_count     = len(arg_names),
-                kw_only_count = kw_only_count,
-                code_name     = code_name,
-                has_starlist  = has_starlist,
-                has_stardict  = has_stardict,
-                source_ref    = source_ref
-            )
-        else:
-            return makeModuleFrame(
-                module     = provider,
-                statements = statements,
-                source_ref = source_ref
-            )
-    else:
-        return StatementsSequence(
-            statements = statements,
-            source_ref = source_ref
+    if provider.isExpressionFunctionBody() or \
+       provider.isExpressionClassBody():
+        result = StatementsFrameFunction(
+            statements  = statements,
+            code_object = code_object,
+            source_ref  = source_ref
         )
+    elif provider.isExpressionGeneratorObjectBody():
+        result = StatementsFrameGenerator(
+            statements  = statements,
+            code_object = code_object,
+            source_ref  = source_ref
+        )
+    elif provider.isExpressionCoroutineObjectBody():
+        result = StatementsFrameCoroutine(
+            statements  = statements,
+            code_object = code_object,
+            source_ref  = source_ref
+        )
+    elif provider.isExpressionAsyncgenObjectBody():
+        result = StatementsFrameAsyncgen(
+            statements  = statements,
+            code_object = code_object,
+            source_ref  = source_ref
+        )
+    else:
+        assert False, provider
+
+    return result
 
 
 def makeStatementsSequenceOrStatement(statements, source_ref):
@@ -277,14 +460,13 @@ def makeStatementsSequenceFromStatements(*statements):
     )
 
 
-
 def makeSequenceCreationOrConstant(sequence_kind, elements, source_ref):
     # Sequence creation. Tries to avoid creations with only constant
     # elements. Would be caught by optimization, but would be useless churn. For
     # mutable constants we cannot do it though.
 
     # Due to the many sequence types, there is a lot of cases here
-    # pylint: disable=R0912
+    # pylint: disable=too-many-branches
 
     for element in elements:
         if not element.isExpressionConstantRef():
@@ -304,10 +486,13 @@ def makeSequenceCreationOrConstant(sequence_kind, elements, source_ref):
             const_type = list
         elif sequence_kind == "SET":
             const_type = set
+
+            if needsSetLiteralReverseInsertion():
+                elements = tuple(reversed(elements))
         else:
             assert False, sequence_kind
 
-        result = ExpressionConstantRef(
+        result = makeConstantRefNode(
             constant      = const_type(
                 element.getConstant()
                 for element in
@@ -328,7 +513,7 @@ def makeSequenceCreationOrConstant(sequence_kind, elements, source_ref):
                 source_ref = source_ref
             )
         elif sequence_kind == "SET":
-            result = ExpressionMakeSet(
+            result = ExpressionMakeSetLiteral(
                 elements   = elements,
                 source_ref = source_ref
             )
@@ -343,13 +528,13 @@ def makeSequenceCreationOrConstant(sequence_kind, elements, source_ref):
     return result
 
 
-def makeDictCreationOrConstant(keys, values, lazy_order, source_ref):
+def makeDictCreationOrConstant(keys, values, source_ref):
     # Create dictionary node. Tries to avoid it for constant values that are not
     # mutable.
 
     assert len(keys) == len(values)
     for key, value in zip(keys, values):
-        if not key.isExpressionConstantRef():
+        if not key.isExpressionConstantRef() or not key.isKnownToBeHashable():
             constant = False
             break
 
@@ -365,22 +550,21 @@ def makeDictCreationOrConstant(keys, values, lazy_order, source_ref):
         # Unless told otherwise, create the dictionary in its full size, so
         # that no growing occurs and the constant becomes as similar as possible
         # before being marshaled.
-        result = ExpressionConstantRef(
+        result = makeConstantRefNode(
             constant      = Constants.createConstantDict(
-                lazy_order = not lazy_order,
-                keys       = [
+                keys   = [
                     key.getConstant()
                     for key in
                     keys
                 ],
-                values     = [
+                values = [
                     value.getConstant()
                     for value in
                     values
                 ]
             ),
-            source_ref    = source_ref,
-            user_provided = True
+            user_provided = True,
+            source_ref    = source_ref
         )
     else:
         result = ExpressionMakeDict(
@@ -393,7 +577,6 @@ def makeDictCreationOrConstant(keys, values, lazy_order, source_ref):
                 for key, value in
                 zip(keys, values)
             ],
-            lazy_order = lazy_order,
             source_ref = source_ref
         )
 
@@ -436,6 +619,17 @@ def makeReraiseExceptionStatement(source_ref):
     )
 
 
+def makeAbsoluteImportNode(module_name, source_ref):
+    return ExpressionBuiltinImport(
+        name        = makeConstantRefNode(module_name, source_ref, True),
+        globals_arg = None,
+        locals_arg  = None,
+        fromlist    = None,
+        level       = makeConstantRefNode(0, source_ref, True),
+        source_ref  = source_ref
+    )
+
+
 def mangleName(variable_name, owner):
     if not variable_name.startswith("__") or variable_name.endswith("__"):
         return variable_name
