@@ -23,10 +23,10 @@ source code comments with developer manual sections.
 """
 
 from nuitka.nodes.AssignNodes import (
-    ExpressionTargetTempVariableRef,
-    ExpressionTargetVariableRef,
     StatementAssignmentVariable,
+    StatementAssignmentVariableName,
     StatementDelVariable,
+    StatementDelVariableName,
     StatementReleaseVariable
 )
 from nuitka.nodes.AttributeNodes import (
@@ -44,7 +44,7 @@ from nuitka.nodes.ComparisonNodes import ExpressionComparisonIsNOT
 from nuitka.nodes.ConditionalNodes import StatementConditional
 from nuitka.nodes.ConstantRefNodes import ExpressionConstantEllipsisRef
 from nuitka.nodes.ContainerOperationNodes import ExpressionListOperationPop
-from nuitka.nodes.OperatorNodes import ExpressionOperationBinaryInplace
+from nuitka.nodes.OperatorNodes import makeExpressionOperationBinaryInplace
 from nuitka.nodes.SliceNodes import (
     ExpressionBuiltinSlice,
     ExpressionSliceLookup,
@@ -58,6 +58,7 @@ from nuitka.nodes.SubscriptNodes import (
 )
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTempVariableRef,
+    ExpressionVariableNameRef,
     ExpressionVariableRef
 )
 from nuitka.PythonVersions import python_version
@@ -73,6 +74,7 @@ from .Helpers import (
     makeStatementsSequenceOrStatement,
     mangleName
 )
+from .ReformulationImportStatements import getFutureSpec
 from .ReformulationTryFinallyStatements import makeTryFinallyStatement
 from .SyntaxErrors import raiseSyntaxError
 
@@ -121,15 +123,13 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source,
                                          source_ref):
     # This is using many variable names on purpose, so as to give names to the
     # unpacked detail values, and has many branches due to the many cases
-    # dealt with, pylint: disable=R0912,R0914
+    # dealt with, pylint: disable=too-many-branches,too-many-locals
 
     if kind == "Name":
-        variable_ref = detail
-
-        return StatementAssignmentVariable(
-            variable_ref = variable_ref,
-            source       = source,
-            source_ref   = source_ref
+        return StatementAssignmentVariableName(
+            variable_name = detail,
+            source        = source,
+            source_ref    = source_ref
         )
     elif kind == "Attribute":
         lookup_source, attribute_name = detail
@@ -215,28 +215,22 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source,
                 statements.insert(
                     starred_index+1,
                     StatementAssignmentVariable(
-                        variable_ref = ExpressionTargetTempVariableRef(
-                            variable   = element_var,
-                            source_ref = source_ref
-                        ),
-                        source       = ExpressionListOperationPop(
+                        variable   = element_var,
+                        source     = ExpressionListOperationPop(
                             list_arg   = ExpressionTempVariableRef(
                                 variable   = starred_list_var,
                                 source_ref = source_ref
                             ),
                             source_ref = source_ref
                         ),
-                        source_ref   = source_ref
+                        source_ref = source_ref
                     )
                 )
             elif element[0] != "Starred":
                 statements.append(
                     StatementAssignmentVariable(
-                        variable_ref = ExpressionTargetTempVariableRef(
-                            variable   = element_var,
-                            source_ref = source_ref
-                        ),
-                        source       = ExpressionSpecialUnpack(
+                        variable   = element_var,
+                        source     = ExpressionSpecialUnpack(
                             value      = ExpressionTempVariableRef(
                                 variable   = source_iter_var,
                                 source_ref = source_ref
@@ -245,7 +239,7 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source,
                             expected   = len(detail),
                             source_ref = source_ref
                         ),
-                        source_ref   = source_ref
+                        source_ref = source_ref
                     )
                 )
             else:
@@ -254,18 +248,15 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source,
 
                 statements.append(
                     StatementAssignmentVariable(
-                        variable_ref = ExpressionTargetTempVariableRef(
-                            variable   = element_var,
-                            source_ref = source_ref
-                        ),
-                        source       = ExpressionBuiltinList(
+                        variable   = element_var,
+                        source     = ExpressionBuiltinList(
                             value      = ExpressionTempVariableRef(
                                 variable   = source_iter_var,
                                 source_ref = source_ref
                             ),
                             source_ref = source_ref
                         ),
-                        source_ref   = source_ref
+                        source_ref = source_ref
                     )
                 )
 
@@ -284,15 +275,12 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source,
 
         statements = [
             StatementAssignmentVariable(
-                variable_ref = ExpressionTargetTempVariableRef(
-                    variable   = source_iter_var,
-                    source_ref = source_ref
-                ),
-                source       = ExpressionBuiltinIter1(
+                variable   = source_iter_var,
+                source     = ExpressionBuiltinIter1(
                     value      = source,
                     source_ref = source_ref
                 ),
-                source_ref   = source_ref
+                source_ref = source_ref
             ),
             makeTryFinallyStatement(
                 provider   = provider,
@@ -332,12 +320,9 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source,
             # usage.
             statements.append(
                 StatementDelVariable(
-                    variable_ref = ExpressionTargetTempVariableRef(
-                        variable   = element_var,
-                        source_ref = source_ref
-                    ),
-                    tolerant     = True,
-                    source_ref   = source_ref,
+                    variable   = element_var,
+                    tolerant   = True,
+                    source_ref = source_ref,
                 )
             )
 
@@ -391,16 +376,13 @@ def buildAssignmentStatements(provider, node, source, source_ref,
 
 def decodeAssignTarget(provider, node, source_ref, allow_none = False):
     # Many cases to deal with, because of the different assign targets,
-    # pylint: disable=R0911,R0912
+    # pylint: disable=too-many-branches,too-many-return-statements
 
     if node is None and allow_none:
         return None
 
     if type(node) is str:
-        return "Name", ExpressionTargetVariableRef(
-            variable_name = mangleName(node, provider),
-            source_ref    = source_ref
-        )
+        return "Name", mangleName(node, provider)
 
     kind = getKind(node)
 
@@ -408,10 +390,7 @@ def decodeAssignTarget(provider, node, source_ref, allow_none = False):
         assert getKind(node.ctx) in ("Store", "Del")
 
     if kind == "Name":
-        return kind, ExpressionTargetVariableRef(
-            variable_name = mangleName(node.id, provider),
-            source_ref    = source_ref
-        )
+        return kind, mangleName(node.id, provider)
     elif kind == "Attribute":
         return kind, (
             buildNode(provider, node.value, source_ref),
@@ -513,12 +492,9 @@ def buildAssignNode(provider, node, source_ref):
 
         statements = [
             StatementAssignmentVariable(
-                variable_ref = ExpressionTargetTempVariableRef(
-                    variable   = tmp_source,
-                    source_ref = source_ref
-                ),
-                source       = source,
-                source_ref   = source_ref
+                variable   = tmp_source,
+                source     = source,
+                source_ref = source_ref
             )
         ]
 
@@ -545,9 +521,13 @@ def buildAssignNode(provider, node, source_ref):
             source_ref = source_ref
         )
 
-# Python3.6 annotation assignment
+
 def buildAnnAssignNode(provider, node, source_ref):
-    if provider.isExpressionClassBody():
+    """ Python3.6 annotation assignment.
+
+    """
+
+    if provider.isCompiledPythonModule() or provider.isExpressionClassBody():
         provider.markAsNeedsAnnotationsDictionary()
 
     # Evaluate the right hand side first, so it can get names provided
@@ -568,7 +548,7 @@ def buildAnnAssignNode(provider, node, source_ref):
 
         # Only name referencing annotations are effective right now.
         if statements[-1].isStatementAssignmentVariable():
-            variable_name = statements[-1].getTargetVariableRef().getVariableName()
+            variable_name = statements[-1].getVariableName()
         else:
             variable_name = None
     else:
@@ -580,11 +560,12 @@ def buildAnnAssignNode(provider, node, source_ref):
         )
 
         if kind == "Name":
-            variable_name = detail.getVariableName()
+            variable_name = detail
         else:
             variable_name = None
 
-    # Only annoations for modules and classes are really made.
+    # Only annotations for modules and classes are really made, for functions
+    # they are ignored like comments.
     if variable_name is not None:
         if provider.isExpressionFunctionBody():
             provider.getVariableForAssignment(variable_name)
@@ -594,9 +575,8 @@ def buildAnnAssignNode(provider, node, source_ref):
             statements.append(
                 StatementAssignmentSubscript(
                     expression = ExpressionVariableRef(
-                        variable_name = "__annotations__",
-                        variable      = provider.getVariableForAssignment("__annotations__"),
-                        source_ref    = source_ref
+                        variable   = provider.getVariableForAssignment("__annotations__"),
+                        source_ref = source_ref
                     ),
                     subscript  = makeConstantRefNode(
                         constant   = variable_name,
@@ -614,17 +594,15 @@ def buildAnnAssignNode(provider, node, source_ref):
     )
 
 
-def buildDeleteStatementFromDecoded(node, kind, detail, source_ref):
+def buildDeleteStatementFromDecoded(kind, detail, source_ref):
     if kind in ("Name", "Name_Exception"):
         # Note: Name_Exception is a "del" for exception handlers that doesn't
         # insist on the variable being defined, user code may do it too, and
         # that will be fine, so make that tolerant.
-        variable_ref = detail
-
-        return StatementDelVariable(
-            variable_ref = variable_ref,
-            tolerant     = kind == "Name_Exception",
-            source_ref   = source_ref.atColumnNumber(node.col_offset+1)
+        return StatementDelVariableName(
+            variable_name = detail,
+            tolerant      = kind == "Name_Exception",
+            source_ref    = source_ref
         )
     elif kind == "Attribute":
         lookup_source, attribute_name = detail
@@ -671,7 +649,6 @@ def buildDeleteStatementFromDecoded(node, kind, detail, source_ref):
         for sub_node in detail:
             result.append(
                 buildDeleteStatementFromDecoded(
-                    node       = node,
                     kind       = sub_node[0],
                     detail     = sub_node[1],
                     source_ref = source_ref
@@ -705,7 +682,6 @@ def buildDeleteNode(provider, node, source_ref):
 
         statements.append(
             buildDeleteStatementFromDecoded(
-                node       = target,
                 kind       = kind,
                 detail     = detail,
                 source_ref = source_ref
@@ -718,14 +694,12 @@ def buildDeleteNode(provider, node, source_ref):
     )
 
 
-def _buildInplaceAssignVariableNode(variable_ref, operator, expression,
+def _buildInplaceAssignVariableNode(variable_name, operator, expression,
                                     source_ref):
-    assert variable_ref.isExpressionTargetVariableRef(), variable_ref
-
-    inplace_node = ExpressionOperationBinaryInplace(
+    inplace_node = makeExpressionOperationBinaryInplace(
         operator   = operator,
-        left       = ExpressionVariableRef(
-            variable_name = variable_ref.getVariableName(),
+        left       = ExpressionVariableNameRef(
+            variable_name = variable_name,
             source_ref    = source_ref
         ),
         right      = expression,
@@ -734,44 +708,33 @@ def _buildInplaceAssignVariableNode(variable_ref, operator, expression,
 
     inplace_node.markAsInplaceSuspect()
 
-    result = (
-        StatementAssignmentVariable(
-            variable_ref = ExpressionTargetVariableRef(
-                variable_name = variable_ref.getVariableName(),
-                source_ref    = source_ref
-            ),
-            source       = inplace_node,
-            source_ref   = source_ref
+    return (
+        StatementAssignmentVariableName(
+            variable_name = variable_name,
+            source        = inplace_node,
+            source_ref    = source_ref
         ),
     )
 
-    return result
-
 
 def _buildInplaceAssignAttributeNode(provider, lookup_source, attribute_name,
                                      tmp_variable1, tmp_variable2, operator,
                                      expression, source_ref):
     # First assign the target value to a temporary variable.
     preserve_to_tmp = StatementAssignmentVariable(
-        variable_ref = ExpressionTargetTempVariableRef(
-            variable   = tmp_variable1,
-            source_ref = source_ref
-        ),
-        source       = ExpressionAttributeLookup(
+        variable   = tmp_variable1,
+        source     = ExpressionAttributeLookup(
             source         = lookup_source.makeClone(),
             attribute_name = attribute_name,
             source_ref     = source_ref
         ),
-        source_ref   = source_ref
+        source_ref = source_ref
     )
 
     # Second assign the in-place result to a temporary variable
     inplace_to_tmp = StatementAssignmentVariable(
-        variable_ref = ExpressionTargetTempVariableRef(
-            variable   = tmp_variable2,
-            source_ref = source_ref
-        ),
-        source       = ExpressionOperationBinaryInplace(
+        variable   = tmp_variable2,
+        source     = makeExpressionOperationBinaryInplace(
             operator   = operator,
             left       = ExpressionTempVariableRef(
                 variable   = tmp_variable1,
@@ -780,7 +743,7 @@ def _buildInplaceAssignAttributeNode(provider, lookup_source, attribute_name,
             right      = expression,
             source_ref = source_ref
         ),
-        source_ref   = source_ref
+        source_ref = source_ref
     )
 
     # Third, copy it over, if the reference values change, i.e. IsNot is true.
@@ -844,21 +807,15 @@ def _buildInplaceAssignSubscriptNode(provider, subscribed, subscript,
                                      expression, source_ref):
     # First assign the subscribed value to a temporary variable.
     preserve_to_tmp1 = StatementAssignmentVariable(
-        variable_ref = ExpressionTargetTempVariableRef(
-            variable   = tmp_variable1,
-            source_ref = source_ref
-        ),
-        source       = subscribed,
-        source_ref   = source_ref
+        variable   = tmp_variable1,
+        source     = subscribed,
+        source_ref = source_ref
     )
     # Second assign the subscript value to a temporary variable
     preserve_to_tmp2  = StatementAssignmentVariable(
-        variable_ref = ExpressionTargetTempVariableRef(
-            variable   = tmp_variable2,
-            source_ref = source_ref
-        ),
-        source       = subscript,
-        source_ref   = source_ref
+        variable   = tmp_variable2,
+        source     = subscript,
+        source_ref = source_ref
     )
 
     execute_in_place = StatementAssignmentSubscript(
@@ -870,7 +827,7 @@ def _buildInplaceAssignSubscriptNode(provider, subscribed, subscript,
             variable   = tmp_variable2,
             source_ref = source_ref
         ),
-        source     = ExpressionOperationBinaryInplace(
+        source     = makeExpressionOperationBinaryInplace(
             operator   = operator,
             left       = ExpressionSubscriptLookup(
                 subscribed = ExpressionTempVariableRef(
@@ -920,16 +877,13 @@ def _buildInplaceAssignSliceNode(provider, lookup_source, lower, upper,
 
     # Due to the 3 inputs, which we need to also put into temporary variables,
     # there are too many variables here, but they are needed.
-    # pylint: disable=R0914
+    # pylint: disable=too-many-locals
 
     # First assign the target value, lower and upper to temporary variables.
     copy_to_tmp = StatementAssignmentVariable(
-        variable_ref = ExpressionTargetTempVariableRef(
-            variable   = tmp_variable1,
-            source_ref = source_ref
-        ),
-        source       = lookup_source,
-        source_ref   = source_ref
+        variable   = tmp_variable1,
+        source     = lookup_source,
+        source_ref = source_ref
     )
 
     final_statements = [
@@ -943,12 +897,9 @@ def _buildInplaceAssignSliceNode(provider, lookup_source, lower, upper,
     if lower is not None:
         statements.append(
             StatementAssignmentVariable(
-                variable_ref = ExpressionTargetTempVariableRef(
-                    variable   = tmp_variable2,
-                    source_ref = source_ref
-                ),
-                source       = lower,
-                source_ref   = source_ref
+                variable   = tmp_variable2,
+                source     = lower,
+                source_ref = source_ref
             )
         )
         final_statements.append(
@@ -974,12 +925,9 @@ def _buildInplaceAssignSliceNode(provider, lookup_source, lower, upper,
     if upper is not None:
         statements.append(
             StatementAssignmentVariable(
-                variable_ref = ExpressionTargetTempVariableRef(
-                    variable   = tmp_variable3,
-                    source_ref = source_ref
-                ),
-                source       = upper,
-                source_ref   = source_ref
+                variable   = tmp_variable3,
+                source     = upper,
+                source_ref = source_ref
             )
         )
         final_statements.append(
@@ -1018,7 +966,7 @@ def _buildInplaceAssignSliceNode(provider, lookup_source, lower, upper,
                     step       = None,
                     source_ref = source_ref
                 ),
-                source     = ExpressionOperationBinaryInplace(
+                source     = makeExpressionOperationBinaryInplace(
                     operator   = operator,
                     left       = ExpressionSubscriptLookup(
                         subscribed = ExpressionTempVariableRef(
@@ -1048,7 +996,7 @@ def _buildInplaceAssignSliceNode(provider, lookup_source, lower, upper,
                 ),
                 lower      = lower_ref1,
                 upper      = upper_ref1,
-                source     = ExpressionOperationBinaryInplace(
+                source     = makeExpressionOperationBinaryInplace(
                     operator   = operator,
                     left       = ExpressionSliceLookup(
                         expression = ExpressionTempVariableRef(
@@ -1080,11 +1028,11 @@ def _buildInplaceAssignSliceNode(provider, lookup_source, lower, upper,
 def buildInplaceAssignNode(provider, node, source_ref):
     # There are many inplace assignment variables, and the detail is unpacked
     # into names, so we end up with a lot of variables, which is on purpose,
-    # pylint: disable=R0914
+    # pylint: disable=too-many-locals
 
     operator = getKind(node.op)
 
-    if operator == "Div" and source_ref.getFutureSpec().isFutureDivision():
+    if operator == "Div" and getFutureSpec().isFutureDivision():
         operator = "TrueDiv"
 
     operator = 'I' + operator
@@ -1098,13 +1046,11 @@ def buildInplaceAssignNode(provider, node, source_ref):
     )
 
     if kind == "Name":
-        variable_ref = detail
-
         statements = _buildInplaceAssignVariableNode(
-            variable_ref = variable_ref,
-            operator     = operator,
-            expression   = expression,
-            source_ref   = source_ref
+            variable_name = detail,
+            operator      = operator,
+            expression    = expression,
+            source_ref    = source_ref
         )
     elif kind == "Attribute":
         lookup_source, attribute_name = detail
