@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,8 +23,6 @@ source code comments with developer manual sections.
 """
 
 from nuitka.nodes.AssignNodes import (
-    ExpressionTargetTempVariableRef,
-    ExpressionTempVariableRef,
     StatementAssignmentVariable,
     StatementReleaseVariable
 )
@@ -39,14 +37,16 @@ from nuitka.nodes.ConditionalNodes import (
     ExpressionConditional,
     StatementConditional
 )
-from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka.nodes.ConstantRefNodes import (
+    ExpressionConstantNoneRef,
+    makeConstantRefNode
+)
 from nuitka.nodes.ExceptionNodes import StatementRaiseException
 from nuitka.nodes.ExecEvalNodes import StatementExec, StatementLocalsDictSync
-from nuitka.nodes.GlobalsLocalsNodes import (
-    ExpressionBuiltinGlobals,
-    ExpressionBuiltinLocals
-)
+from nuitka.nodes.GlobalsLocalsNodes import ExpressionBuiltinGlobals
+from nuitka.nodes.NodeMakingHelpers import makeExpressionBuiltinLocals
 from nuitka.nodes.TypeNodes import ExpressionBuiltinIsinstance
+from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
 
 from .Helpers import (
     buildNode,
@@ -58,13 +58,6 @@ from .Helpers import (
 from .ReformulationTryFinallyStatements import makeTryFinallyStatement
 
 
-def _getLocalsClassNode(provider):
-    if provider.isCompiledPythonModule():
-        return ExpressionBuiltinGlobals
-    else:
-        return ExpressionBuiltinLocals
-
-
 def wrapEvalGlobalsAndLocals(provider, globals_node, locals_node,
                              temp_scope, source_ref):
     """ Wrap the locals and globals arguments for "eval".
@@ -84,14 +77,12 @@ def wrapEvalGlobalsAndLocals(provider, globals_node, locals_node,
     )
 
     if locals_node is None:
-        locals_node = ExpressionConstantRef(
-            constant   = None,
+        locals_node = ExpressionConstantNoneRef(
             source_ref = source_ref
         )
 
     if globals_node is None:
-        globals_node = ExpressionConstantRef(
-            constant   = None,
+        globals_node = ExpressionConstantNoneRef(
             source_ref = source_ref
         )
 
@@ -126,8 +117,7 @@ def wrapEvalGlobalsAndLocals(provider, globals_node, locals_node,
                 variable   = globals_keeper_variable,
                 source_ref = source_ref
             ),
-            right      = ExpressionConstantRef(
-                constant   = None,
+            right      = ExpressionConstantNoneRef(
                 source_ref = source_ref
             ),
             source_ref = source_ref
@@ -136,7 +126,8 @@ def wrapEvalGlobalsAndLocals(provider, globals_node, locals_node,
             variable   = globals_keeper_variable,
             source_ref = source_ref
         ),
-        expression_yes = _getLocalsClassNode(provider)(
+        expression_yes = makeExpressionBuiltinLocals(
+            provider   = provider,
             source_ref = source_ref
         ),
         source_ref     = source_ref
@@ -145,20 +136,14 @@ def wrapEvalGlobalsAndLocals(provider, globals_node, locals_node,
     pre_statements = [
         # First assign globals and locals temporary the values given.
         StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = globals_keeper_variable,
-                source_ref = source_ref
-            ),
-            source       = globals_node,
-            source_ref   = source_ref,
+            variable   = globals_keeper_variable,
+            source     = globals_node,
+            source_ref = source_ref,
         ),
         StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = locals_keeper_variable,
-                source_ref = source_ref
-            ),
-            source       = locals_node,
-            source_ref   = source_ref,
+            variable   = locals_keeper_variable,
+            source     = locals_node,
+            source_ref = source_ref,
         ),
         StatementConditional(
             condition  = ExpressionComparisonIs(
@@ -166,20 +151,16 @@ def wrapEvalGlobalsAndLocals(provider, globals_node, locals_node,
                     variable   = locals_keeper_variable,
                     source_ref = source_ref
                 ),
-                right      = ExpressionConstantRef(
-                    constant   = None,
+                right      = ExpressionConstantNoneRef(
                     source_ref = source_ref
                 ),
                 source_ref = source_ref
             ),
             yes_branch = makeStatementsSequenceFromStatement(
                 StatementAssignmentVariable(
-                    variable_ref = ExpressionTargetTempVariableRef(
-                        variable   = locals_keeper_variable,
-                        source_ref = source_ref
-                    ),
-                    source       = locals_default,
-                    source_ref   = source_ref,
+                    variable   = locals_keeper_variable,
+                    source     = locals_default,
+                    source_ref = source_ref,
                 )
             ),
             no_branch  = None,
@@ -191,22 +172,18 @@ def wrapEvalGlobalsAndLocals(provider, globals_node, locals_node,
                     variable   = globals_keeper_variable,
                     source_ref = source_ref
                 ),
-                right      = ExpressionConstantRef(
-                    constant   = None,
+                right      = ExpressionConstantNoneRef(
                     source_ref = source_ref
                 ),
                 source_ref = source_ref
             ),
             yes_branch = makeStatementsSequenceFromStatement(
                 StatementAssignmentVariable(
-                    variable_ref = ExpressionTargetTempVariableRef(
-                        variable   = globals_keeper_variable,
+                    variable   = globals_keeper_variable,
+                    source     = ExpressionBuiltinGlobals(
                         source_ref = source_ref
                     ),
-                    source       = ExpressionBuiltinGlobals(
-                        source_ref = source_ref
-                    ),
-                    source_ref   = source_ref,
+                    source_ref = source_ref,
                 )
             ),
             no_branch  = None,
@@ -239,7 +216,7 @@ def buildExecNode(provider, node, source_ref):
     # This is using many variables, due to the many details this is
     # dealing with. The locals and globals need to be dealt with in
     # temporary variables, and we need handling of indicators, so
-    # that is just the complexity, pylint: disable=R0914
+    # that is just the complexity, pylint: disable=too-many-locals
 
     exec_globals = node.globals
     exec_locals = node.locals
@@ -264,7 +241,7 @@ def buildExecNode(provider, node, source_ref):
                     exception_name = "TypeError",
                     source_ref     = source_ref
                 ),
-                exception_value = ExpressionConstantRef(
+                exception_value = makeConstantRefNode(
                     constant   = """\
 exec: arg 1 must be a string, file, or code object""",
                     source_ref = source_ref
@@ -275,8 +252,6 @@ exec: arg 1 must be a string, file, or code object""",
             )
 
     if not provider.isCompiledPythonModule():
-        provider.markAsExecContaining()
-
         if orig_globals is None:
             provider.markAsUnqualifiedExecContaining(source_ref)
 
@@ -285,16 +260,14 @@ exec: arg 1 must be a string, file, or code object""",
     locals_value  = buildNode(provider, exec_locals, source_ref, True)
 
     if locals_value is None:
-        locals_value = ExpressionConstantRef(
-            constant   = None,
+        locals_value = ExpressionConstantNoneRef(
             source_ref = source_ref
         )
 
     globals_value = buildNode(provider, exec_globals, source_ref, True)
 
     if globals_value is None:
-        globals_value = ExpressionConstantRef(
-            constant   = None,
+        globals_value = ExpressionConstantNoneRef(
             source_ref = source_ref
         )
 
@@ -323,43 +296,31 @@ exec: arg 1 must be a string, file, or code object""",
     tried = (
         # First evaluate the source code expressions.
         StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = source_variable,
-                source_ref = source_ref
-            ),
-            source       = source_code,
-            source_ref   = source_ref
+            variable   = source_variable,
+            source     = source_code,
+            source_ref = source_ref
         ),
         # Assign globals and locals temporary the values given, then fix it
         # up, taking note in the "plain" temporary variable, if it was an
         # "exec" statement with None arguments, in which case the copy back
         # will be necessary.
         StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = globals_keeper_variable,
-                source_ref = source_ref
-            ),
-            source       = globals_value,
-            source_ref   = source_ref
+            variable   = globals_keeper_variable,
+            source     = globals_value,
+            source_ref = source_ref
         ),
         StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = locals_keeper_variable,
-                source_ref = source_ref
-            ),
-            source       = locals_value,
-            source_ref   = source_ref
+            variable   = locals_keeper_variable,
+            source     = locals_value,
+            source_ref = source_ref
         ),
         StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = plain_indicator_variable,
-                source_ref = source_ref
-            ),
-            source       = ExpressionConstantRef(
+            variable   = plain_indicator_variable,
+            source     = makeConstantRefNode(
                 constant   = False,
                 source_ref = source_ref
             ),
-            source_ref   = source_ref
+            source_ref = source_ref
         ),
         StatementConditional(
             condition  = ExpressionComparisonIs(
@@ -367,22 +328,18 @@ exec: arg 1 must be a string, file, or code object""",
                     variable   = globals_keeper_variable,
                     source_ref = source_ref
                 ),
-                right      = ExpressionConstantRef(
-                    constant   = None,
+                right      = ExpressionConstantNoneRef(
                     source_ref = source_ref
                 ),
                 source_ref = source_ref
             ),
             yes_branch = makeStatementsSequenceFromStatements(
                 StatementAssignmentVariable(
-                    variable_ref = ExpressionTargetTempVariableRef(
-                        variable   = globals_keeper_variable,
+                    variable   = globals_keeper_variable,
+                    source     = ExpressionBuiltinGlobals(
                         source_ref = source_ref
                     ),
-                    source       = ExpressionBuiltinGlobals(
-                        source_ref = source_ref
-                    ),
-                    source_ref   = source_ref,
+                    source_ref = source_ref,
                 ),
                 StatementConditional(
                     condition  = ExpressionComparisonIs(
@@ -390,33 +347,27 @@ exec: arg 1 must be a string, file, or code object""",
                             variable   = locals_keeper_variable,
                             source_ref = source_ref
                         ),
-                        right      = ExpressionConstantRef(
-                            constant   = None,
+                        right      = ExpressionConstantNoneRef(
                             source_ref = source_ref
                         ),
                         source_ref = source_ref
                     ),
                     yes_branch = makeStatementsSequenceFromStatements(
                         StatementAssignmentVariable(
-                            variable_ref = ExpressionTargetTempVariableRef(
-                                variable   = locals_keeper_variable,
+                            variable   = locals_keeper_variable,
+                            source     = makeExpressionBuiltinLocals(
+                                provider   = provider,
                                 source_ref = source_ref
                             ),
-                            source       = _getLocalsClassNode(provider)(
-                                source_ref = source_ref
-                            ),
-                            source_ref   = source_ref,
+                            source_ref = source_ref,
                         ),
                         StatementAssignmentVariable(
-                            variable_ref = ExpressionTargetTempVariableRef(
-                                variable   = plain_indicator_variable,
-                                source_ref = source_ref
-                            ),
-                            source       = ExpressionConstantRef(
+                            variable   = plain_indicator_variable,
+                            source     = makeConstantRefNode(
                                 constant   = True,
                                 source_ref = source_ref
                             ),
-                            source_ref   = source_ref,
+                            source_ref = source_ref,
                         )
                     ),
                     no_branch  = None,
@@ -430,23 +381,19 @@ exec: arg 1 must be a string, file, or code object""",
                             variable   = locals_keeper_variable,
                             source_ref = source_ref
                         ),
-                        right      = ExpressionConstantRef(
-                            constant   = None,
+                        right      = ExpressionConstantNoneRef(
                             source_ref = source_ref
                         ),
                         source_ref = source_ref
                     ),
                     yes_branch = makeStatementsSequenceFromStatement(
                         statement = StatementAssignmentVariable(
-                            variable_ref = ExpressionTargetTempVariableRef(
-                                variable   = locals_keeper_variable,
-                                source_ref = source_ref
-                            ),
-                            source       = ExpressionTempVariableRef(
+                            variable   = locals_keeper_variable,
+                            source     = ExpressionTempVariableRef(
                                 variable   = globals_keeper_variable,
                                 source_ref = source_ref
                             ),
-                            source_ref   = source_ref,
+                            source_ref = source_ref,
                         )
                     ),
                     no_branch  = None,
@@ -471,11 +418,8 @@ exec: arg 1 must be a string, file, or code object""",
             ),
             yes_branch = makeStatementsSequenceFromStatement(
                 statement = StatementAssignmentVariable(
-                    variable_ref = ExpressionTargetTempVariableRef(
-                        variable   = source_variable,
-                        source_ref = source_ref
-                    ),
-                    source       = ExpressionCallEmpty(
+                    variable   = source_variable,
+                    source     = ExpressionCallEmpty(
                         called     = ExpressionAttributeLookup(
                             source         = ExpressionTempVariableRef(
                                 variable   = source_variable,
@@ -486,7 +430,7 @@ exec: arg 1 must be a string, file, or code object""",
                         ),
                         source_ref = source_ref
                     ),
-                    source_ref   = source_ref
+                    source_ref = source_ref
                 )
             ),
             no_branch  = None,
@@ -515,7 +459,7 @@ exec: arg 1 must be a string, file, or code object""",
                         variable   = plain_indicator_variable,
                         source_ref = source_ref
                     ),
-                    right      = ExpressionConstantRef(
+                    right      = makeConstantRefNode(
                         constant   = True,
                         source_ref = source_ref
                     ),
