@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,20 +23,23 @@ source code comments with developer manual sections.
 """
 
 from nuitka.nodes.AssignNodes import (
-    ExpressionTargetTempVariableRef,
-    ExpressionTargetVariableRef,
     StatementAssignmentVariable,
+    StatementAssignmentVariableName,
     StatementReleaseVariable
 )
+from nuitka.nodes.AsyncgenNodes import (
+    ExpressionAsyncgenObjectBody,
+    ExpressionMakeAsyncgenObject
+)
 from nuitka.nodes.BuiltinIteratorNodes import (
     ExpressionBuiltinIter1,
-    ExpressionSpecialUnpack,
     StatementSpecialUnpackCheck
 )
-from nuitka.nodes.BuiltinRefNodes import ExpressionBuiltinRef
+from nuitka.nodes.BuiltinNextNodes import ExpressionSpecialUnpack
+from nuitka.nodes.BuiltinRefNodes import makeExpressionBuiltinRef
 from nuitka.nodes.CallNodes import ExpressionCallNoKeywords
 from nuitka.nodes.CodeObjectSpecs import CodeObjectSpec
-from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
 from nuitka.nodes.ContainerMakingNodes import ExpressionMakeTuple
 from nuitka.nodes.CoroutineNodes import (
     ExpressionCoroutineObjectBody,
@@ -51,21 +54,20 @@ from nuitka.nodes.FunctionNodes import (
 from nuitka.nodes.GeneratorNodes import (
     ExpressionGeneratorObjectBody,
     ExpressionMakeGeneratorObject,
-    StatementGeneratorReturn
+    StatementGeneratorReturnNone
 )
 from nuitka.nodes.ParameterSpecs import ParameterSpec
-from nuitka.nodes.ReturnNodes import StatementReturn
+from nuitka.nodes.ReturnNodes import StatementReturn, StatementReturnNone
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTempVariableRef,
-    ExpressionVariableRef
+    ExpressionVariableNameRef
 )
 from nuitka.PythonVersions import python_version
-from nuitka.tree import SyntaxErrors
 
 from .Helpers import (
+    buildFrameNode,
     buildNode,
     buildNodeList,
-    buildStatementsNode,
     detectFunctionBodyKind,
     extractDocFromBody,
     getKind,
@@ -74,22 +76,13 @@ from .Helpers import (
     mangleName
 )
 from .ReformulationTryFinallyStatements import makeTryFinallyStatement
+from .SyntaxErrors import raiseSyntaxError
 
 
-def _insertFinalReturnStatement(function_statements_body, return_class,
-                                source_ref):
-    return_statement = return_class(
-        expression = ExpressionConstantRef(
-            constant      = None,
-            user_provided = True,
-            source_ref    = source_ref
-        ),
-        source_ref = source_ref
-    )
-
+def _insertFinalReturnStatement(function_statements_body, return_statement):
     if function_statements_body is None:
         function_statements_body = makeStatementsSequenceFromStatement(
-            statement = return_statement,
+            statement = return_statement
         )
     elif not function_statements_body.isStatementAborting():
         function_statements_body.setStatements(
@@ -103,14 +96,13 @@ def _insertFinalReturnStatement(function_statements_body, return_class,
 
 
 def buildFunctionNode(provider, node, source_ref):
-    # Functions have way too many details, pylint: disable=R0914
+    # Functions have way too many details, pylint: disable=too-many-branches,too-many-locals
 
     assert getKind(node) == "FunctionDef"
 
     function_statement_nodes, function_doc = extractDocFromBody(node)
 
-    function_kind, flags, _written_variables, _non_local_declarations, _global_declarations = \
-      detectFunctionBodyKind(
+    function_kind, flags = detectFunctionBodyKind(
         nodes = function_statement_nodes
     )
 
@@ -133,6 +125,7 @@ def buildFunctionNode(provider, node, source_ref):
             flags      = flags,
             source_ref = source_ref
         )
+        code_body.qualname_provider = provider
 
         for variable in function_body.getVariables():
             code_body.getVariableForReference(variable.getName())
@@ -175,7 +168,7 @@ def buildFunctionNode(provider, node, source_ref):
         source_ref    = source_ref
     )
 
-    function_statements_body = buildStatementsNode(
+    function_statements_body = buildFrameNode(
         provider    = code_body,
         nodes       = function_statement_nodes,
         code_object = code_object,
@@ -186,8 +179,9 @@ def buildFunctionNode(provider, node, source_ref):
         # TODO: Generators might have to raise GeneratorExit instead.
         function_statements_body = _insertFinalReturnStatement(
             function_statements_body = function_statements_body,
-            return_class             = StatementReturn,
-            source_ref               = source_ref
+            return_statement         = StatementReturnNone(
+                source_ref = source_ref
+            )
         )
 
     if function_statements_body.isStatementsFrame():
@@ -218,15 +212,35 @@ def buildFunctionNode(provider, node, source_ref):
     # CPython made these optional, but secretly applies them when it does
     # "class __new__".  We add them earlier, so our optimization will see it.
     if node.name == "__new__" and \
-       not decorators and \
        provider.isExpressionClassBody():
 
-        decorators = (
-            ExpressionBuiltinRef(
-                builtin_name = "staticmethod",
-                source_ref   = source_ref
-            ),
-        )
+        for decorator in decorators:
+            if decorator.isExpressionVariableNameRef() and \
+               decorator.getVariableName() == "staticmethod":
+                break
+        else:
+            decorators.append(
+                makeExpressionBuiltinRef(
+                    builtin_name = "staticmethod",
+                    source_ref   = source_ref
+                )
+            )
+
+    if python_version >= 360 and \
+       node.name == "__init_subclass__" and \
+       provider.isExpressionClassBody():
+
+        for decorator in decorators:
+            if decorator.isExpressionVariableNameRef() and \
+               decorator.getVariableName() == "classmethod":
+                break
+        else:
+            decorators.append(
+                makeExpressionBuiltinRef(
+                    builtin_name = "classmethod",
+                    source_ref   = source_ref
+                )
+            )
 
     decorated_function = function_creation
     for decorator in decorators:
@@ -239,44 +253,57 @@ def buildFunctionNode(provider, node, source_ref):
             source_ref = decorator.getSourceReference()
         )
 
-    result = StatementAssignmentVariable(
-        variable_ref = ExpressionTargetVariableRef(
-            variable_name = mangleName(node.name, provider),
-            source_ref    = source_ref
-        ),
-        source       = decorated_function,
-        source_ref   = source_ref
+    result = StatementAssignmentVariableName(
+        variable_name = mangleName(node.name, provider),
+        source        = decorated_function,
+        source_ref    = source_ref
     )
 
     if python_version >= 340:
-        function_body.qualname_setup = result.getTargetVariableRef()
+        function_body.qualname_setup = result.getVariableName()
 
     return result
 
 
 def buildAsyncFunctionNode(provider, node, source_ref):
     # We are creating a function here that creates coroutine objects, with
-    # many details each, pylint: disable=R0914
+    # many details each, pylint: disable=too-many-locals
     assert getKind(node) == "AsyncFunctionDef"
 
     function_statement_nodes, function_doc = extractDocFromBody(node)
 
+    function_kind, flags = detectFunctionBodyKind(
+        nodes       = function_statement_nodes,
+        start_value = "Coroutine"
+    )
+
     creator_function_body, _, code_object = buildFunctionWithParsing(
         provider      = provider,
-        function_kind = "Coroutine",
+        function_kind = function_kind,
         name          = node.name,
-        flags         = set(),
+        flags         = (),
         function_doc  = function_doc,
         node          = node,
         source_ref    = source_ref
     )
 
-    function_body = ExpressionCoroutineObjectBody(
-        provider   = creator_function_body,
-        name       = node.name,
-        flags      = set(),
-        source_ref = source_ref
-    )
+    if function_kind == "Coroutine":
+        function_body = ExpressionCoroutineObjectBody(
+            provider   = creator_function_body,
+            name       = node.name,
+            flags      = flags,
+            source_ref = source_ref
+        )
+    else:
+        function_body = ExpressionAsyncgenObjectBody(
+            provider   = creator_function_body,
+            name       = node.name,
+            flags      = flags,
+            source_ref = source_ref
+        )
+
+    for variable in creator_function_body.getVariables():
+        function_body.getVariableForReference(variable.getName())
 
     decorators = buildNodeList(
         provider   = provider,
@@ -290,7 +317,7 @@ def buildAsyncFunctionNode(provider, node, source_ref):
         source_ref = source_ref
     )
 
-    function_statements_body = buildStatementsNode(
+    function_statements_body = buildFrameNode(
         provider    = function_body,
         nodes       = function_statement_nodes,
         code_object = code_object,
@@ -299,8 +326,9 @@ def buildAsyncFunctionNode(provider, node, source_ref):
 
     function_statements_body = _insertFinalReturnStatement(
         function_statements_body = function_statements_body,
-        return_class             = StatementGeneratorReturn,
-        source_ref               = source_ref
+        return_statement         = StatementGeneratorReturnNone(
+            source_ref = source_ref
+        )
     )
 
     if function_statements_body.isStatementsFrame():
@@ -321,17 +349,30 @@ def buildAsyncFunctionNode(provider, node, source_ref):
         source_ref    = source_ref
     )
 
+    if function_kind == "Coroutine":
+        creation_node = ExpressionMakeCoroutineObject(
+            coroutine_ref = ExpressionFunctionRef(
+                function_body = function_body,
+                source_ref    = source_ref
+            ),
+            code_object   = code_object,
+            source_ref    = source_ref
+        )
+    else:
+        creation_node = ExpressionMakeAsyncgenObject(
+            asyncgen_ref = ExpressionFunctionRef(
+                function_body = function_body,
+                source_ref    = source_ref
+            ),
+            code_object  = code_object,
+            source_ref   = source_ref
+        )
+
+
     creator_function_body.setBody(
         makeStatementsSequenceFromStatement(
             statement = StatementReturn(
-                expression = ExpressionMakeCoroutineObject(
-                    coroutine_ref = ExpressionFunctionRef(
-                        function_body = function_body,
-                        source_ref    = source_ref
-                    ),
-                    code_object   = code_object,
-                    source_ref    = source_ref
-                ),
+                expression = creation_node,
                 source_ref = source_ref
             )
         )
@@ -361,16 +402,17 @@ def buildAsyncFunctionNode(provider, node, source_ref):
         )
 
 
-    result = StatementAssignmentVariable(
-        variable_ref = ExpressionTargetVariableRef(
-            variable_name = mangleName(node.name, provider),
-            source_ref    = source_ref
-        ),
-        source       = decorated_function,
-        source_ref   = source_ref
+    result = StatementAssignmentVariableName(
+        variable_name = mangleName(node.name, provider),
+        source        = decorated_function,
+        source_ref    = source_ref
     )
 
-    function_body.qualname_setup = result.getTargetVariableRef()
+    function_body.qualname_setup = result.getVariableName()
+
+    # Share the non-local declarations. TODO: This may also apply to generators
+    # and async generators.
+    creator_function_body.non_local_declarations = function_body.non_local_declarations
 
     return result
 
@@ -390,7 +432,7 @@ def buildParameterKwDefaults(provider, node, function_body, source_ref):
               zip(kw_only_names, node.args.kw_defaults):
                 if kw_default is not None:
                     keys.append(
-                        ExpressionConstantRef(
+                        makeConstantRefNode(
                             constant   = kw_only_name,
                             source_ref = source_ref
                         )
@@ -413,13 +455,12 @@ def buildParameterKwDefaults(provider, node, function_body, source_ref):
 
 
 def buildParameterAnnotations(provider, node, source_ref):
-    # Too many branches, because there is too many cases, pylint: disable=R0912
+    # Too many branches, because there is too many cases, pylint: disable=too-many-branches
 
     # Build annotations. We are hiding here, that it is a Python3 only feature.
     if python_version < 300:
         return None
 
-
     # Starting with Python 3.4, the names of parameters are mangled in
     # annotations as well.
     if python_version < 340:
@@ -432,7 +473,7 @@ def buildParameterAnnotations(provider, node, source_ref):
 
     def addAnnotation(key, value):
         keys.append(
-            ExpressionConstantRef(
+            makeConstantRefNode(
                 constant      = mangle(key),
                 source_ref    = source_ref,
                 user_provided = True
@@ -450,8 +491,8 @@ def buildParameterAnnotations(provider, node, source_ref):
                     value = buildNode(provider, arg.annotation, source_ref)
                 )
         elif getKind(arg) == "Tuple":
-            for arg in arg.elts:
-                extractArg(arg)
+            for sub_arg in arg.elts:
+                extractArg(sub_arg)
         else:
             assert False, getKind(arg)
 
@@ -505,7 +546,7 @@ def buildParameterAnnotations(provider, node, source_ref):
 def buildFunctionWithParsing(provider, function_kind, name, function_doc, flags,
                              node, source_ref):
     # This contains a complex re-formulation for nested parameter functions.
-    # pylint: disable=R0914
+    # pylint: disable=too-many-locals
 
     kind = getKind(node)
 
@@ -545,35 +586,41 @@ def buildFunctionWithParsing(provider, function_kind, name, function_doc, flags,
     normal_args = extractNormalArgs(node.args.args)
 
     parameters = ParameterSpec(
-        name          = name,
-        normal_args   = normal_args,
-        kw_only_args  = [
+        ps_name          = name,
+        ps_normal_args   = normal_args,
+        ps_kw_only_args  = [
             extractArg(arg)
             for arg in
             node.args.kwonlyargs
             ]
               if python_version >= 300 else
             [],
-        list_star_arg = extractArg(node.args.vararg),
-        dict_star_arg = extractArg(node.args.kwarg),
-        default_count = len(node.args.defaults)
+        ps_list_star_arg = extractArg(node.args.vararg),
+        ps_dict_star_arg = extractArg(node.args.kwarg),
+        ps_default_count = len(node.args.defaults)
     )
 
-    message = parameters.checkValid()
+    message = parameters.checkParametersValid()
 
     if message is not None:
-        SyntaxErrors.raiseSyntaxError(
+        raiseSyntaxError(
             message,
-            source_ref
+            source_ref.atColumnNumber(node.col_offset),
         )
 
+    parent_module = provider.getParentModule()
+
     code_object = CodeObjectSpec(
-        code_name     = name,
-        code_kind     = function_kind,
-        arg_names     = parameters.getParameterNames(),
-        kw_only_count = parameters.getKwOnlyParameterCount(),
-        has_starlist  = parameters.getStarListArgumentName() is not None,
-        has_stardict  = parameters.getStarDictArgumentName() is not None
+        co_name           = name,
+        co_kind           = function_kind,
+        co_varnames       = parameters.getParameterNames(),
+        co_argcount       = parameters.getArgumentCount(),
+        co_kwonlyargcount = parameters.getKwOnlyParameterCount(),
+        co_has_starlist   = parameters.getStarListArgumentName() is not None,
+        co_has_stardict   = parameters.getStarDictArgumentName() is not None,
+        co_filename       = parent_module.getRunTimeFilename(),
+        co_lineno         = source_ref.getLineNumber(),
+        future_spec       = parent_module.getFutureSpec()
     )
 
     outer_body = ExpressionFunctionBody(
@@ -604,15 +651,12 @@ def buildFunctionWithParsing(provider, function_kind, name, function_doc, flags,
 
             statements.append(
                 StatementAssignmentVariable(
-                    variable_ref = ExpressionTargetTempVariableRef(
-                        variable   = iter_var,
-                        source_ref = source_ref
-                    ),
-                    source       = ExpressionBuiltinIter1(
+                    variable   = iter_var,
+                    source     = ExpressionBuiltinIter1(
                         value      = source,
                         source_ref = source_ref
                     ),
-                    source_ref   = source_ref
+                    source_ref = source_ref
                 )
             )
 
@@ -624,11 +668,8 @@ def buildFunctionWithParsing(provider, function_kind, name, function_doc, flags,
 
                     statements.append(
                         StatementAssignmentVariable(
-                            variable_ref = ExpressionTargetTempVariableRef(
-                                variable   = arg_var,
-                                source_ref = source_ref
-                            ),
-                            source       = ExpressionSpecialUnpack(
+                            variable   = arg_var,
+                            source     = ExpressionSpecialUnpack(
                                 value      = ExpressionTempVariableRef(
                                     variable   = iter_var,
                                     source_ref = source_ref
@@ -637,7 +678,7 @@ def buildFunctionWithParsing(provider, function_kind, name, function_doc, flags,
                                 expected   = len(arg_names),
                                 source_ref = source_ref
                             ),
-                            source_ref   = source_ref
+                            source_ref = source_ref
                         )
                     )
 
@@ -682,7 +723,7 @@ def buildFunctionWithParsing(provider, function_kind, name, function_doc, flags,
 
         for arg_name in parameters.getParameterNames():
             if arg_name.startswith('.'):
-                source = ExpressionVariableRef(
+                source = ExpressionVariableNameRef(
                     variable_name = arg_name,
                     source_ref    = source_ref
                 )
@@ -692,7 +733,7 @@ def buildFunctionWithParsing(provider, function_kind, name, function_doc, flags,
                 )
             else:
                 values.append(
-                    ExpressionVariableRef(
+                    ExpressionVariableNameRef(
                         variable_name = arg_name,
                         source_ref    = source_ref
                     )
@@ -701,12 +742,12 @@ def buildFunctionWithParsing(provider, function_kind, name, function_doc, flags,
                 inner_arg_names.append(arg_name)
 
         inner_parameters = ParameterSpec(
-            name          = inner_name,
-            normal_args   = inner_arg_names,
-            kw_only_args  = (),
-            list_star_arg = None,
-            dict_star_arg = None,
-            default_count = None
+            ps_name          = inner_name,
+            ps_normal_args   = inner_arg_names,
+            ps_kw_only_args  = (),
+            ps_list_star_arg = None,
+            ps_dict_star_arg = None,
+            ps_default_count = None
         )
 
         function_body = ExpressionFunctionBody(
@@ -742,7 +783,7 @@ def buildFunctionWithParsing(provider, function_kind, name, function_doc, flags,
         outer_body.setBody(
             makeStatementsSequenceFromStatement(
                 statement = makeTryFinallyStatement(
-                    provider,
+                    provider   = outer_body,
                     tried      = statements,
                     final      = [
                         StatementReleaseVariable(
@@ -750,8 +791,11 @@ def buildFunctionWithParsing(provider, function_kind, name, function_doc, flags,
                             source_ref = source_ref
                         )
                         for variable in
-                        outer_body.getTempVariables()
-                    ]   ,
+                        sorted(
+                            outer_body.getTempVariables(),
+                            key = lambda variable: variable.getName()
+                        )
+                    ],
                     source_ref = source_ref,
                     public_exc = False
                 )
@@ -767,8 +811,8 @@ def addFunctionVariableReleases(function):
     assert function.isExpressionFunctionBody() or \
            function.isExpressionClassBody() or \
            function.isExpressionGeneratorObjectBody() or \
-           function.isExpressionCoroutineObjectBody()
-
+           function.isExpressionCoroutineObjectBody() or \
+           function.isExpressionAsyncgenObjectBody()
 
     releases = []
 
