@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -20,21 +20,19 @@ Namespace packages of Python3.3
 
 """
 
+import os
+
 from nuitka import Options
-from nuitka.nodes.AssignNodes import (
-    ExpressionTargetVariableRef,
-    StatementAssignmentVariable
-)
+from nuitka.nodes.AssignNodes import StatementAssignmentVariableName
 from nuitka.nodes.AttributeNodes import ExpressionAttributeLookup
 from nuitka.nodes.CallNodes import ExpressionCallNoKeywords
-from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
 from nuitka.nodes.ContainerMakingNodes import (
     ExpressionMakeList,
     ExpressionMakeTuple
 )
 from nuitka.nodes.FutureSpecs import FutureSpec
 from nuitka.nodes.ImportNodes import (
-    ExpressionImportModule,
     ExpressionImportModuleHard,
     ExpressionImportName
 )
@@ -44,17 +42,21 @@ from nuitka.nodes.ModuleNodes import (
 )
 from nuitka.PythonVersions import python_version
 from nuitka.SourceCodeReferences import SourceCodeReference
-from nuitka.utils.Utils import dirname
 
-from .Helpers import makeStatementsSequenceFromStatement
+from .Helpers import (
+    makeAbsoluteImportNode,
+    makeStatementsSequenceFromStatement
+)
 from .VariableClosure import completeVariableClosures
 
 
 def createPathAssignment(source_ref):
     if Options.getFileReferenceMode() == "original":
-        path_value = ExpressionConstantRef(
+        path_value = makeConstantRefNode(
             constant      = [
-                dirname(source_ref.getFilename())
+                os.path.dirname(
+                    source_ref.getFilename()
+                )
             ],
             source_ref    = source_ref,
             user_provided = True
@@ -86,36 +88,28 @@ def createPathAssignment(source_ref):
             source_ref = source_ref
         )
 
-    return  StatementAssignmentVariable(
-        variable_ref = ExpressionTargetVariableRef(
-            variable_name = "__path__",
-            source_ref    = source_ref
-        ),
-        source       = path_value,
-        source_ref   = source_ref
+    return StatementAssignmentVariableName(
+        variable_name = "__path__",
+        source        = path_value,
+        source_ref    = source_ref
     )
 
 
 def createPython3NamespacePath(package_name, module_relpath, source_ref):
-    return StatementAssignmentVariable(
-        variable_ref = ExpressionTargetVariableRef(
-            variable_name = "__path__",
-            source_ref    = source_ref
-        ),
-        source       = ExpressionCallNoKeywords(
+    return StatementAssignmentVariableName(
+        variable_name = "__path__",
+        source        = ExpressionCallNoKeywords(
             called     = ExpressionImportName(
-                module      = ExpressionImportModule(
+                module      = makeAbsoluteImportNode(
                     module_name = "_frozen_importlib"
                                     if python_version < 350 else
                                   "_frozen_importlib_external",
-                    import_list = (),
-                    level       = 0,
                     source_ref  = source_ref
                 ),
                 import_name = "_NamespacePath",
                 source_ref  = source_ref
             ),
-            args       = ExpressionConstantRef(
+            args       = makeConstantRefNode(
                 constant   = (
                     package_name,
                     [module_relpath],
@@ -125,7 +119,7 @@ def createPython3NamespacePath(package_name, module_relpath, source_ref):
             ),
             source_ref =  source_ref
         ),
-        source_ref   = source_ref
+        source_ref    = source_ref
     )
 
 
@@ -133,16 +127,17 @@ def createNamespacePackage(package_name, module_relpath):
     parts = package_name.split('.')
 
     source_ref = SourceCodeReference.fromFilenameAndLine(
-        filename    = module_relpath,
-        line        = 1,
-        future_spec = FutureSpec(),
+        filename = module_relpath,
+        line     = 1
     )
     source_ref = source_ref.atInternal()
 
     package_package_name = '.'.join(parts[:-1]) or None
     package = CompiledPythonPackage(
         name         = parts[-1],
+        mode         = "compiled",
         package_name = package_package_name,
+        future_spec  = FutureSpec(),
         source_ref   = source_ref,
     )
 
