@@ -19,7 +19,8 @@
 #     limitations under the License.
 #
 
-import os, sys, subprocess
+import os
+import sys
 
 # The test runner needs "lxml" itself.
 try:
@@ -28,25 +29,28 @@ except ImportError:
     print("Warning, no 'lxml' module installed, cannot do XML based tests.")
     sys.exit(0)
 
-# Find common code relative in file system. Not using packages for test stuff.
+# Find nuitka package relative to us.
 sys.path.insert(
     0,
     os.path.normpath(
         os.path.join(
             os.path.dirname(os.path.abspath(__file__)),
+            "..",
             ".."
         )
     )
 )
-from test_common import (
-    my_print,
-    setup,
+
+from nuitka.tools.testing.Common import (  # isort:skip
+    check_output,
     convertUsing2to3,
     createSearchMode,
     hasModule,
-    check_output
+    my_print,
+    setup
 )
 
+
 python_version = setup()
 
 # The Python version used by Nuitka needs "lxml" too.
@@ -122,7 +126,7 @@ def checkSequence(statements):
                     if called_expression.attrib["builtin_name"] == "print":
                         continue
 
-        if kind == "Frame":
+        if kind == "FrameModule":
             checkSequence(getRole(statement, "statements"))
 
             continue
@@ -179,7 +183,7 @@ for filename in sorted(os.listdir('.')):
 
             # Coverage modules hates Nuitka to re-execute, and so we must avoid
             # that.
-            python_path = subprocess.check_output(
+            python_path = check_output(
                 [
                     os.environ["PYTHON"],
                     "-c"
