@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -20,6 +20,6 @@ def f():
   try:
     raise A
   except:
-    print "caught"
+    print("caught")
   except A:
-    print "hit"
+    print("hit")
